## ######### PROJECT NAME : ##########
## 
## raid.pm
## 
## ######### PROJECT DESCRIPTION : ###
## 
## 
## 
## ###################################
## 
## Made by Pujos Sylvain
## Login   <spujos@intellique.com>
## 
## Started on  Wed Jul 13 12:01:00 2005 Pujos Sylvain
## Last update Thu Aug 28 17:43:24 2008 Boutonnet Alexandre
## 
## ###################################
## Version = 1.0

package raid;

use strict;
use Data::Dumper;

use base 'Exporter';
our @EXPORT_OK = qw( alarms bbu cache capacity createunit deleteport deleteunit state info list_ctl
      list_port list_port_creatable list_port_rebuildable list_unit
      list_unit_rebuildable rebuild rescan smart verify );

use Lib_Disk;

use lib "/usr/local/lib/site_perl/raid";

use raid_logiciel;
use raid_materiel;
use raid_lvm;
use lib_raid;


# Nom des controleurs
# Nom a afficher => code interne
my %name = (
		"clo"	=> "name_clo",
		"cm"	=> "name_cm",
		"lvm"	=> "name_lvm",
    );

my %module_list_ctl= (
        clo	=> \&raid_logiciel::list_ctl,
        cm	=> \&raid_materiel::list_ctl,
        lvm	=> \&raid_lvm::list_ctl,
    );

my %module_info_all = (
		clo	=> \&raid_logiciel::info_all,
		cm	=> \&raid_materiel::info_all,
		lvm	=> \&raid_lvm::info_all,
    );

my %module_state = (
		clo	=> \&raid_logiciel::state,
		cm	=> \&raid_materiel::state,
    );

my %module_alarms = (
		clo	=> \&raid_logiciel::alarms,
		cm	=> \&raid_materiel::alarms,
    );

my %module_switch = (
		clo	=> \&raid_logiciel::switch,
		cm	=> \&raid_materiel::switch,
		lvm	=> \&raid_lvm::switch,
    );

# Hash contenant le nombre minimum de parametre par fonction
my %param = (
        "info"			=> 1,
        "state"			=> 1,
        "alarms"        => 1,
        "rescan"        => 1,
        "deleteport"    => 2,
        "deleteunit"    => 2,
        "createunit"    => 3,
        "rebuild"       => 3,
        "verify"        => 2,
        "list_unit"		=> 1,
        "list_port"		=> 1,
        "smart"			=> 1,
        "bbu"			=> 1,
        "cache"			=> 2,
        "capacity"      => 1,
    );

#############################################

# Analyse les parametres et appel le module

#############################################

sub verif_and_call {
    # Recoit le nom de la commande ainsi que  tout les arguments, appel la fonction qui va bien

    # Je verifie que le nombre de parametre ne soit pas inferieur a un minimum
    return error( "Not enough arguments" ) if ( @_ <= $param{$_[0]} );

    # Verification du type de raid
    my @tab_ctl = split( "_", $_[1] );

    return error( "Invalid controler name" ) unless ( exists $module_switch{ $tab_ctl[0] } );
    return $module_switch{ $tab_ctl[0] }->( @_ );
}


#############################################

# Fonction necessitant d appeler tout 
# les sous modules

#############################################

sub list_ctl {
    # Recuperation de la liste des controleurs sur tous les raids.
    my @list;

    my %name_inv = map { $name{ $_ } => $_ } keys %name;
    foreach $_ ( keys %module_list_ctl ) {
	my ($etat, $retour) = $module_list_ctl{ $_ }->( $name_inv{ "name_" . $_ } );
	push @list, @{ $retour} unless ( $etat );

	return error_and_stop( $retour ) if ( $etat == -1 );
    }
	
    return ok( \@list );
}

sub info {
    # Recuperation des informations sur tous les raids.

    # Un parametre est fourni
    return verif_and_call("info", @_) if (@_ > 0);
	
    # Aucun parametre n'est fourni
    my ($etat, $ref_list_ctl) = boucle_tout_controleur( \%module_info_all );
		
    return error_and_stop( $ref_list_ctl ) if ( $etat == -1 );
    return ok( $ref_list_ctl );
}

sub alarms {
    # Un parametre est fourni
    return verif_and_call("alarms", @_) if (@_ > 0);

    # Aucun parametre n'est fourni
    my ($etat, $ref_list_ctl) = list_ctl();
    return ok( "No controler found" ) unless ( @{ $ref_list_ctl} );

    my $ref_info;
    foreach ( @{ $ref_list_ctl} ) {
		my $name_ctl		= $_;
		my @tab_type_ctl	= split("_",$_);
		my $type_ctl		= $tab_type_ctl[0];

		if ( exists $module_alarms{ $type_ctl } ) {
		    my ($etat_info_all, $ref_info_all) = $module_alarms{ $type_ctl }->( $name_ctl );
		    $ref_info .= $ref_info_all if ( $ref_info_all );
		}
    }

    return ok( $ref_info );
}

sub state {
    # Un parametre est fourni
    return verif_and_call("state", @_) if (@_ > 0);

    # Aucun parametre n'est fourni
    my ($etat, $ref_list_ctl) = list_ctl();
    return ok( "No controler found" ) unless ( @{ $ref_list_ctl} );

    my $ref_info = 0;
    my $etat_info = 0;
    foreach ( @{ $ref_list_ctl } ) {
	my $name_ctl		= $_;
	my @tab_type_ctl	= split("_",$_);
	my $type_ctl		= $tab_type_ctl[0];

	if ( exists $module_alarms{ $type_ctl } ) {
	    my ($etat_info_all, $ref_info_all) = $module_state{ $type_ctl }->( $name_ctl );
	    $ref_info	+= $ref_info_all if ( $ref_info_all );
	    $etat_info	+= $etat_info_all;
	}
    }

    return error( $ref_info ) if ( $etat_info);
    return ok( $ref_info );
}

#############################################

# Fonctions d aide au developpement

#############################################

sub list_unit_rebuildable {
    # Recuperation de la liste des units rebuildable.
    my ( $etat_info, $ref_info ) = info();
    my @retour;

    return error( $ref_info ) if ( $etat_info);

    foreach my $ctl ( keys %{ $ref_info } ) {
        foreach my $unit ( keys %{ $ref_info->{ $ctl }{'Unit'} } ) {
            if ( $ref_info->{ $ctl }{'Unit'}{ $unit }{'Status'} and $ref_info->{ $ctl }{'Unit'}{ $unit }{'Status'} ne 'OK' ) {
                push @retour, "$ctl $unit";
            }
        }
    }

    return ok( \@retour );
}

sub list_port_rebuildable {
    # Recuperation de la liste des ports dispo pour la reconsttruction.
    my ( $etat_info, $ref_info ) = info();
    my @retour;

    return error( $ref_info ) if ( $etat_info);

    foreach my $ctl ( keys %{ $ref_info } ) {

		if ( $ref_info->{ $ctl }{'Unit'}{ '-' } ) {
			foreach my $port ( @{ $ref_info->{ $ctl }{'Unit'}{ '-' }{'Port'} } ) {
				push @retour, "$ctl $port";
			}

			delete $ref_info->{ $ctl }{'Unit'}{ '-' } ;
		}

		foreach my $unit ( keys %{ $ref_info->{ $ctl }{'Unit'} } ) {
		    if ( $ref_info->{ $ctl }{'Unit'}{ $unit }{'UnitType'} eq 'SPARE' ) {
				foreach my $port ( @{ $ref_info->{ $ctl }{'Unit'}{ $unit }{'Port'} } ) {
			 	   push @retour, "$ctl $port";
				}
		    }
		}
    }

    return ok( \@retour );
}

sub list_port_creatable {
    # Recuperation de la liste des ports dispo pour la reconstruction.
    my ( $etat_info, $ref_info ) = info();
    my @retour;
    my %hash_partition;

    return error( $ref_info ) if ( $etat_info);

    delete $ref_info->{'clo'};

    my @tab;

    my ($ret_code, $ret_tab) = Lib_Disk::get_disk_list();
    return error( $ret_tab ) if ($ret_code);

    foreach my $disk ( @{$ret_tab} )
    {
	my ($ret_code, $ret_part) = Lib_Disk::get_disk_partition( $disk );
	
    	next if ( $ret_code );
	
	push @tab, @{$ret_part};
    }

    map { push @retour, "clo $_" } @tab ;
    map { push @retour, "lvm $_" } @tab ;

    foreach my $ctl ( keys %{ $ref_info } ) {
        next unless $ref_info->{ $ctl }{'Unit'}{ '-' } ;

        foreach my $port ( @{ $ref_info->{ $ctl }{'Unit'}{ '-' }{'Port'} } ) {
            push @retour, "$ctl $port";
        }
    }

    return ok( \@retour );
}

#############################################

# Fonctions necessitant de connaitre le 
# controleur

#############################################

sub list_unit {
# Recuperation de la liste des units sur tout un controleur.
    return verif_and_call("list_unit", @_);
}

sub list_port {
# Recuperation de la liste des ports sur tout un controleur.
    return verif_and_call("list_port", @_);
}


sub bbu {
    return verif_and_call("bbu", @_);
}

sub cache {
    return verif_and_call("cache", @_);
}

sub capacity {
    return verif_and_call("capacity", @_);
}

sub createunit {
    return verif_and_call("createunit", @_);
}

sub deleteport {
    return verif_and_call("deleteport", @_);
}

sub deleteunit {
    return verif_and_call("deleteunit", @_);
}

sub rebuild {
    return verif_and_call("rebuild", @_);
}

sub rescan {
    return verif_and_call("rescan", @_);

}

sub verify {
    return verif_and_call("verify", @_);
}

sub smart {
    return verif_and_call("smart", @_);
}


#############################################

# Fonctions locale

#############################################

sub boucle_tout_controleur {
    my $module = shift;

    # Aucun parametre n'est fourni
    my ($etat, $ref_list_ctl) = list_ctl();

    return error_and_stop( $ref_list_ctl ) if ( $etat == -1 );
    return ok( "No controler found" ) unless ( @{ $ref_list_ctl} );

    my $ref_info;
    foreach ( @{ $ref_list_ctl} ) {
	my $name_ctl		= $_;
	my @tab_type_ctl	= split("_",$_);
	my $type_ctl		= $tab_type_ctl[0];

	my ($etat_info_all, $ref_info_all) = $module->{ $type_ctl }->( $name_ctl );
	
	$ref_info->{$name_ctl} = $ref_info_all->{ $name_ctl } if ( $ref_info_all->{ $name_ctl } );
    }

    return ok( $ref_info );
}

1;


=head1 NOM

raid_cli pour raid command line interface.

=head1 DEFINITION

=head2 Generalite

Utilitaire de gestion de raid en ligne de commande. raid_cli est un utilitaire englobant les differentes technologies raid de manieres evolutives (raid logiciel, raid materiel 3ware, raid materiel adaptec...).

=head2 Parametres

Le nom du controleur se compose  de differentes informations separees par des _ :
 - Son type (cm pour materiel, clo pour logiciel).
 - Le nom de son fabricant (3ware, adaptec...).
 - La version fournie par le fabricant (7506-8...).
 - Une indication sur le numero du controleur (pour 3ware, il se note c1, c2 ...).

 Exemple : cm_3ware_7506-8_c1 (controleur c1 materiel d'une carte 3ware 7506-8), clo (controleur logiciel).

=head2 Specificite

=head3 Raid Logiciel

 - Le raid logiciel creer son unite en /dev/md[0-50] en choisissant le 1er de libre 
 - Les partitions utilise pour construire une unite sont appelees ports.
 - Le raid creer est un raid virtuel.

=head1 RAID SUPPORTE

Liste des utilitaires supportes par raid_cli :

 - mdadm.
 - tw_cli (version : 2.00.00.063, 2.00.02.008, 2.00.02.009).

=head1 COMMANDE

=head2 Maintenance

 - createunit controleur raid[0 1 5] ou spare ou single ou jbod ports : 
              Creer une unite du type souhaite en y integrant les ports specifies.

 - deleteunit  controleur unite :
               Supprime l'unite du controleur specifie.

 
 - deleteport controleur port :
               Supprime le port du controleur specifie.

 - rebuild controleur unite port :
               Reconstruit l'unite avec le port specifie.
                .penguin-co
 - rescan controleur :
                le controleur verifie la presence de nouveau ports.

 - verify controleur unite :
               Verifie l'integrite du controleur specifie. 

=head2 Information

 - list_ctl controleur :
              Renvoie la liste des controleurs present sur la machine.
 
 - list_port controleur unite : 
              Renvoie la liste des ports associe a l'unite du controleur specifie.

 - list_unit controleur :
              Renvoie la liste des unites associe au controleur specifie.
 
 - info [controleur unite port]:
              Renvoie les informations sur les differents controleurs en service sur la machine. Si il a une unite ou un port de specifie, les informations seront limite a ce dernier.

 - help :     Affiche une page d'aide sur l'utilisation du raid_cli.

=head2 Surveillance

 - state [controleur] : 
              Renvoie 0 si le controleur n'a pas de soucis et 1 si il y a un probleme. Si le controleur n'est pas specifie, state demandera a tout les controleurs en service leur etat, et renverra 1 si l'un d'eux est defaillant.
 
 - alarms [controleur] :
              Renvoie les alarmes du controleur. Si aucun controleur n'est specifie, alarms renvoie les alarmes de tout les controleurs en service.

=head1 UTILISATION

Cette section se compose d'une suite d'exemples concret sur la mise en place, la maintenance et la suppression de raid.

 1) Affichage des informations sur tout les raid existant sur la machine :

 >raid_cli info

 Ctl   Unit         Port        Type    Model              Etat    Taille    Cmpl
 ********************************************************************************
 
 cm_3ware_7506-8_c1                     7506-8             Optimal
  |--[ u2 ]-------------------- SPARE -------------------- OK      114.49 Go
  |---------------[ p2 ]--------------- Maxtor 6Y120P0 --- OK      114.49 Go
  |--[ u3 ]-------------------- RAID-5 ------------------- OK      228.99 Go
  |---------------[ p3 ]--------------- Maxtor 6Y120P0 --- OK      114.49 Go
  |---------------[ p4 ]--------------- Maxtor 6Y120P0 --- OK      114.49 Go

La machine contient un raid materiel, le controleur est de marque 3ware modele 706-8, il se nomme c1. Ce controleur est compose de 2 unitees (u2 et u3), u3 est un raid5 constitue de 2 ports (p3 et P4) qui sont 2 disques dur Maxtor 6Y120P0 de 120Gigas. L'etat du controleur, du raid et des disques est optimal.

2) Cherchons a en savoir plus sur le port p3

 >raid_cli info p3

 Port   Status           Unit   Size        Blocks        Serial
 ---------------------------------------------------------------
 p3     OK               u3     114.49 GB   240121728     Y3NH2SLE

L'information sur le numero de serie ainsi que son nombre de blocks nous est fourni en plus.

3) Creations d'un raid0 logiciel utilisant les partitions /dev/sda1 et /dev/sdb1

>raid_cli createunit clo raid0 /dev/sda1 /dev/sdb1


4) Suppression de l'unite md0 

>raid_cli deletunit clo /dev/md0

5) Verification de l'etat du controleur cm_3ware_7506-8_c1 

>raid_cli state cm_3ware_7506-8_c1
 Etat: 0
 tout va bien :)

=head1 MISC

raid_cli contient une completion qui permet, en pressant 2 fois sur la touche tabulation, de completer et d'afficher les differentes commandes ou parametres completant la saisie en cours. 

=head1 AUTEUR

Dayan Romain
