#############################################################
##
## Made by Florac Emmanuel
##
## Started on mar dc 16 16:54:41 CET 2008 Florac Emmanuel
##

# nom du package:
package Arcconf_info_v1;

use strict;
use Data::Dumper;
use raid_smart;

my $arcconf_name	= "/usr/sbin/arcconf";

##################################
#recuperation des infos sur la ou les carte(s) presente(s)
#le hash %arcconf est retourne
sub arcconf_info_ctl {
	my $ctl = shift;

	my @arrayinfo = qx#$arcconf_name getconfig $ctl al #;
	my %arcconf=( $ctl => {} );
	
	my $section = 'none' ;
	my $currentdevice = -1;
	my %status = ( Optimal => 'OK', Online => 'OK', Ready => 'OK', Hot => 'OK', Impacted => 'DEGRADE' );
	
	# variables temporaires pour gagner du temps  la fin
	my %alldrives;
	my %useddrives;
	
	foreach my $info ( @arrayinfo ) {
	
		next if $info =~ /^\s*\-+/;
	
		if  ( $info =~ /^Controller information/ ) { 
			$section = 'controller'  ;
			next;
		}
		if  ( $info =~ /^Logical device information/ ) { 
			$section = 'unit'  ;
			next;
		}
		if  ( $info =~ /^Physical Device information/ ) { 
			$section = 'port'  ;
			next;
		}
		
		if ( $section eq 'controller' ) {
			$arcconf{$ctl}{Model} = $1 if $info =~ /^\s+Controller\sModel\s+:\s+(\w+\s+\w+)/ ;
			$arcconf{$ctl}{NotOpt} = $1 if $info =~ /^\s+Defunct disk drive count\s+:\s+(\w+\s+\w+)/ ;
		}
		
		if ( $section eq 'unit' ) {
			if  ( $info =~ /^Logical device number (\d+)/ ) {
				$currentdevice = "u$1";
				$arcconf{$ctl}{Unit}{$currentdevice} = {VCmpl => '-', Cmpl => '-', Blocks => '-' };
				$arcconf{$ctl}{Units}++ ;
				next;
			}
			
			$arcconf{$ctl}{Unit}{$currentdevice}{Size} = sprintf "%0.2f", $1/1024 
				if $info =~ /^\s+Size\s+:\s+(\d+)\s+MB/ ;
			
			if ( $info =~ /^\s+Status of logical device\s+:\s+(\w+)/ ) {
				$arcconf{$ctl}{Unit}{$currentdevice}{Status} = $status{$1} || $1;
				
				if ( $arcconf{$ctl}{Unit}{$currentdevice}{Status} ne 'OK' ) {
					( $arcconf{$ctl}{Unit}{$currentdevice}{Status},
					$arcconf{$ctl}{Unit}{$currentdevice}{Cmpl}) = 
						_unit_status($ctl,$currentdevice);
				}
				
			}
			$arcconf{$ctl}{Unit}{$currentdevice}{UnitType} = "RAID-$1" 
				if $info =~ /^\s+RAID level\s+:\s+(\w+)/ ;
			# on rcupre les numros de srie pour comparer ensuite avec la liste de drives...
			$arcconf{$ctl}{Unit}{$currentdevice}{"Port$1"} = {}
				if  $info =~ /^\s+Segment\s+\d+\s+:\s+\w+\s+\(\d+,\d+\)\s+(\w+)/ ;
				
			
		}
		
		if ( $section eq 'port' or $section eq 'drive' ) {
			if ( $info =~ /^\s+Device #(\d+)/ ) {
				$arcconf{$ctl}{Ports}++ ;
				$currentdevice = "p$1" ;
				next;
			}
			
			if ( $info =~ /^\s+Device is a Hard drive/ ) {
				$section = 'drive';
				$arcconf{$ctl}{Drives}++ ;
				$arcconf{$ctl}{Drive}{$currentdevice}{Blocks}='-';
				$arcconf{$ctl}{Drive}{$currentdevice}{Unit} ='-';
				$alldrives{$currentdevice} = $currentdevice ;
				next;
			}
			
		}
		
		if ( $section eq 'drive' ) {
			if ( $info =~ /^\s+State\s+:\s+(\w+)/ ) {
				$arcconf{$ctl}{Drive}{$currentdevice}{Status} = $status{$1} || $1;
				if ( $1 eq 'Hot' ) {
					# crer une unit bidon "SPARE"
					$arcconf{$ctl}{Unit}{'sp'} = {VCmpl => '-', Cmpl => '-', Blocks => '-', Status => 'OK', UnitType => 'SPARE', Blocks => '-' };
					$arcconf{$ctl}{Units}++ ;
					# mettre le disque dedans
					$arcconf{$ctl}{Drive}{$currentdevice}{Unit} = 'sp';
					# $arcconf{$ctl}{Unit}{sp}{"Port$1"} = {};
					push @{$arcconf{$ctl}{Unit}{sp}{Port}}, $currentdevice ;
					# le marquer comme "utilis"
					$useddrives{$currentdevice} = $currentdevice ; 
				}
			}

			$arcconf{$ctl}{Drive}{$currentdevice}{Size} = sprintf "%0.2f", $1/1024 
				if $info =~ /^\s+Size\s+:\s+(\w+)\s+MB/;
			$arcconf{$ctl}{Drive}{$currentdevice}{Model} = $1
				if $info =~ /^\s+Model\s+:\s+(\w+)/;

			if ( $info =~ /^\s+Serial number\s+:\s+(\w+)/ ) {
				$arcconf{$ctl}{Drive}{$currentdevice}{Serial} = $1 ;
				# l on matche les drives et les units
				foreach my $unit ( keys %{$arcconf{$ctl}{Unit}} ) {
					if ( exists $arcconf{$ctl}{Unit}{$unit}{"Port$1"} ) {
						delete $arcconf{$ctl}{Unit}{$unit}{"Port$1"} ;
						$arcconf{$ctl}{Drive}{$currentdevice}{Unit} = $unit;
						push @{$arcconf{$ctl}{Unit}{$unit}{Port}}, $currentdevice ;
						$useddrives{$currentdevice} = $currentdevice ;
					}
				}
			}
		}
	}
	
	# ensuite on reparcourt les disques pour retirer ceux qui sont attribus
	foreach my $drive ( keys %useddrives ) {
		delete $alldrives{$drive} ;
	}
	@{$arcconf{$ctl}{Unit}{'-'}{Port}} = ( keys %alldrives );

	return 0, undef unless ( @arrayinfo );
	
	# print Dumper \%arcconf;
	return 0, \%arcconf;
}

##################################
#recuperation de la liste des controleurs presents
sub arcconf_list_ctl {

	# les contrleurs sont numrots en commenant  1
	# pour avoir le nombre de contrleurs on interroge le contrleur 1

	my @retour_info = qx#$arcconf_name getconfig 1 ad#;
	my @list_ctl;
	my ( $ctl_count ) = ( $retour_info[0] =~ /^Controllers\sfound:\s(\d+)/g );
	
	for ( 1 .. $ctl_count ) {
		@retour_info = qx#$arcconf_name getconfig $_ ad#;
		
		foreach my $line (@retour_info) {
			if ( $line =~ /^\s+Controller\sModel\s+:\s+(\w+)\s+(\w+)/ ) {
				push @list_ctl, $2 . "_" . $_  ;
			}
		}
	}
	return 0, \@list_ctl;
}

##################################
#recuperation de la liste des unites pour un controleur donne
sub arcconf_list_unit {
	my $ctl = shift;

	my @liste_unit_port = qx#$arcconf_name -F -g -z rankconfigstruct -p $ctl#;
	my @liste_unit;

	foreach ( @liste_unit_port ) {
		push @liste_unit, "u".$1 if ( $_ =~ m/^Array:\s*(\d+)/ );
	}

 	return 0, \@liste_unit;
}

##################################
#recuperation de la liste des ports pour un controleur donne
sub arcconf_list_port {
	my $ctl = shift;

	my @liste_unit_port = qx#$arcconf_name -D -g -z alldriveinfostruct#;
	my @liste_port;

	foreach ( sort @liste_unit_port ) {
		push @liste_port, "p".$1 if ( $_ =~ m/^SES_SlotId\s*:\s*(\d+)/ );
	}

 	return 0, \@liste_port;
}


##################################
#recuperation des infos smart pour un controleur et un port donne
sub arcconf_smart {
    my ( $ctl, $port ) = @_;

    if ( ! $port ) {
		my ($var, $ref) = arcconf_info_ctl( $ctl );
		return 0, 41 if ( $ref->{$ctl}{"NotOpt"} );
		return 0, 0;
    }
    
    if ($port =~ /^u\d+$/) {
		my ($var, $ref) = arcconf_info_ctl( $ctl );

		return 0, 11 unless ( $ref->{$ctl}{"Unit"}{$port}{"Status"} );
		return 0, raid_smart::translate_text_to_code( $ref->{$ctl}{"Unit"}{$port}{"Status"} );
    }


    if ($port =~ /^p\d+$/) {
		my ($var, $ref) = arcconf_info_ctl( $ctl );

		return 0, 11 unless ( $ref->{$ctl}{"Drive"}{$port}{"Status"} );
		return 0, raid_smart::translate_text_to_code( $ref->{$ctl}{"Drive"}{$port}{"Status"} );
    }

    return 0, "Bad command";
}

## ###################################
## Fonction state :
## 
## state()

sub state {
    my $ctl = shift;
    my ($var, $ref) = arcconf_info_ctl( $ctl );

    return 0, $ref->{$ctl}{"NotOpt"};
}

## ###################################
## Fonction capacity :

sub capacity {
    my $ctl = shift;

	my $raid_support = qx#$arcconf_name  -C -g -z allinfo#;
    return 0, $raid_support;
}

## ###################################
## Fonction bbu :

sub bbu {
	my $ctl = shift;
	my $bbuinfo;
	
	my @ctl_info = qx($arcconf_name  -C -g -z allinfo);
    
	my $found_ctl;
	my $ctlregexp = qr/\*+Controller\s($ctl)/ ;
	
	foreach ( @ctl_info ) {
		if ( m/$ctlregexp/ ) {
			$found_ctl++ ;
			next
		}
		
		if ( $found_ctl ) {
			$bbuinfo .= $_ 
				if m/^Battery/ ;
			
			last if m/^Battery\sHoldup/;
		}
	}
	
	
	return 0, $bbuinfo;
	
}


## ###################################
## Fonction private :
## 
## raid_level

sub _raid_level {
	my $raid_level	= shift;
	my $ctl			= shift;
	my $unit		= shift;
	
	my $span = qx#$arcconf_name  -C -g -z allinfo#;
	chomp $span;

	if ( $raid_level =~ /Primary-(\d+), Secondary-(\d+), RAID Level Qualifier-/ ) {
		if ( $span == 1 ) {
			return "RAID-$1";
		}
		elsif ( $span == 2 ) {
			return "RAID-$1$2";
		}
	}
	
	return "NA";
}

## ###################################
## Fonction interrogation statut RAID

sub _unit_status {
	my $ctl  = shift ;
	my $unit = shift ;
	
	$unit=~ s/u//g;
	
	my @ctl_status = qx($arcconf_name  getstatus $ctl);
	
	my $found = 0;
	my $status='UNKNOWN';
	my $percent=-1;
	
	foreach my $info ( @ctl_status ) {
		if ( $info =~ /^\s+Logical device\s+:\s+($unit)$/ ) {
			$found = 1;
			next;
		}
		
		if ( $info =~ /^\s+Current operation\s+:\s+([\w\/]+)/ ) {
			$found = 1;
			$status=$1;
			next;
		}
		if ( $info =~ /^\s+Percentage complete\s+:\s+([\w\/]+)/ ) {
			$percent=$1;
			last;
		}
		
	}

	return $status, $percent
}

1;
