#############################################################
##
## Made by Florac Emmanuel
##
## Started on  mer sep 24 16:21:10 CEST 2008 Florac Emmanuel
##

# nom du package:
package Lxcr_info_v1;

use strict;
use Data::Dumper;
use raid_smart;

my $Lxcr_name = "/usr/sbin/LXCR";

##################################
#recuperation des infos sur la ou les carte(s) presente(s)
#le hash %Lxcr est retourne
sub Lxcr_info_ctl {
    my $ctl = shift;
    my %Lxcr;

    # temporaire : mauvaise gestion contrleur/baie
    my $enclosure = $ctl + 1;

    # statut final
    my $ctlstatus = 0;

    my @arrayinfo = qx#$Lxcr_name -F -g -z rankconfigstruct -p $ctl #;

    # vrif si contrleur secondaire
    if ( $arrayinfo[0] =~ /Error: Raid system does not exist/ and $ctl > 0 ) {

        # on renvoie l'info du contrleur primaire
        my $otherctl = $ctl - 1;
        my ( $error, $data ) = Lxcr_info_ctl($otherctl);
        %{ $data->{"$ctl"} } = %{ $data->{"$otherctl"} };
        delete $data->{$otherctl};
        return ( $error, $data );
    }

    my @alldiskinfo = qx#$Lxcr_name  -D -g -z alldriveinfostruct #;
    return 0, undef unless (@arrayinfo);

    # parcours des Arrays
    my $currentarray;
    my $currentarraydata;

    foreach (@arrayinfo) {
        if (
m/^Array:(\d+),\s*RAID\s*(\d+),\s*(\d+)\s*Drives\s*\(.*\),*\s\w+\sChunk,\s*(\d+)\s*\wB/
          )
        {
            if ($currentarray) {
                $Lxcr{$ctl}{Unit}{$currentarray} = $currentarraydata;
            }

            $currentarray = "u$1";
            $currentarraydata =
              { Size => $4, UnitType => "RAID-$2", Blocks => $4 };
            $Lxcr{$ctl}{Units}++;
        }

        # valeurs possibles :Initialized Initializing Trusted
        # Trusted = Not Optimal!
        #if ( m/^Array\sStatus:\s(\w+)/ ) {
        #	$currentarraydata->{Status} = $1;
        #}

# valeurs possibles : Rebuilding Array / Not Fault Tolerant /
# RAID5 Not Initialized / Rank Invalidated / Expanding / Fault Warning / Fault Tolerant
        if (m/^Fault\sStatus:\s([\w\s]+)\n/) {
            $currentarraydata->{Status} =
              ( $1 eq 'Fault Tolerant' ? 'OK' : $1 );
        }
    }
    if ($currentarray) {
        $Lxcr{$ctl}{Unit}{$currentarray} = $currentarraydata;
    }

    # parcours des drives et filtrage par l'enclosure
    my $currentdrive;
    my $currentdrivedata;

    foreach (@alldiskinfo) {
        if (m/^NumDrives\s+:\s+(\d+)/) {
            $Lxcr{$ctl}{Drives} = $1;
        }

        if (m/^NoOfEntries\s*:\s+(\d+)/) {
            $Lxcr{$ctl}{Ports} = $1;
        }

        if (m/^\*+\s+Entry\s+(\d+)/) {
            if ($currentdrive) {
                $Lxcr{$ctl}{Drive}{$currentdrive} = $currentdrivedata;
                undef $currentdrivedata;
            }
            $currentdrive = "p$1";
        }

        # fitrage par l'enclosure
        if (m/^EnclosureNo\s+:\s+(\d+)/) {
            $currentdrivedata->{Enclosure} = $1;
        }

        if ( $currentdrivedata->{Enclosure} = $enclosure ) {
            if (m/^DriveStatus\s+:\s+(\d+)/) {
                $currentdrivedata->{Status} = ( $1 == 0 ? 'OK' : 'FAILED' );
                $ctlstatus++ if $currentdrivedata->{Status} eq 'FAILED';
            }

            if (m/^Capacity\s+:\s+(\d+)/) {
                $currentdrivedata->{Size} = $1;
            }

            if (m/^ProductId\s+:\s+(\w+)/) {
                $currentdrivedata->{Model} = $1;
            }

            if (m/^SerialNumber\s+:\s+(\w+)/) {
                $currentdrivedata->{Serial} = $1;
            }

            if (m/^RankNo\s+:\s+(\d+)/) {
                my $unit =
                  ( $1 == 0xFF ? '-' : ( $1 == 0xF5 ? 'SPARE' : "u$1" ) );
                $currentdrivedata->{Unit} = $unit;
                push @{ $Lxcr{$ctl}{Unit}{$unit}{Port} }, $currentdrive;
            }

        }
    }

    if ($currentdrivedata) {
        $Lxcr{$ctl}{Drive}{$currentdrive} = $currentdrivedata;
    }

    $Lxcr{$ctl}{NotOpt} = $ctlstatus;
    $Lxcr{$ctl}{Model}  = _ctlmodel($ctl);

    my ( undef, @tab_bbu ) = bbu($ctl);
    foreach (@tab_bbu) {
        if (m/Battery Status\s+:\s+(.+)/) {
            $Lxcr{$ctl}{'BBU'}{'Status'} = ( $1 eq "Good" ? "OK" : $1 );
        }

        $Lxcr{$ctl}{'BBU'}{'Hours'} = $1
          if (m/Battery Holdup Time\s+:\s+(\d+)\shours/);

        if (m/Battery Capacity\s+:\s+(\d+)%/) {
            $Lxcr{$ctl}{'BBU'}{'BBUReady'} = ( $1 > 60 ? "Yes" : "No" );
        }
    }

    #print Dumper \%Lxcr;
    return 0, \%Lxcr;
}

##################################
#recuperation de la liste des controleurs presents
sub Lxcr_list_ctl {
    my @retour_info = qx#$Lxcr_name  -C -g -z allinfo#;
    my @list_ctl;

    my $conttype;

    my $typeregexp = qr/^\w+\stype\sis\s(\w+)/;
    my $ctlregexp  = qr/\*+Controller\s(\d+)/;

    foreach (@retour_info) {
        if (m/$typeregexp/) {
            $conttype = $1;
        }
        if (m/$ctlregexp/) {
            my $ctlnum = $1;
            $list_ctl[$ctlnum] = $conttype . '_' . $ctlnum;
        }
    }

    return 0, \@list_ctl;
}

##################################
#recuperation de la liste des unites pour un controleur donne
sub Lxcr_list_unit {
    my $ctl = shift;

    my @liste_unit_port = qx#$Lxcr_name -F -g -z rankconfigstruct -p $ctl#;
    my @liste_unit;

    foreach (@liste_unit_port) {
        push @liste_unit, "u" . $1 if ( $_ =~ m/^Array:\s*(\d+)/ );

    }

    return 0, \@liste_unit;
}

##################################
#recuperation de la liste des ports pour un controleur donne
sub Lxcr_list_port {
    my $ctl = shift;

    my @liste_unit_port = qx#$Lxcr_name -D -g -z alldriveinfostruct#;
    my @liste_port;

    foreach ( sort @liste_unit_port ) {
        push @liste_port, "p" . $1 if ( $_ =~ m/^SES_SlotId\s*:\s*(\d+)/ );
    }

    return 0, \@liste_port;
}

##################################
#recuperation des infos smart pour un controleur et un port donne
sub Lxcr_smart {
    my ( $ctl, $port ) = @_;

    if ( !$port ) {
        my ( $var, $ref ) = Lxcr_info_ctl($ctl);
        return 0, 41 if ( $ref->{$ctl}{"NotOpt"} );
        return 0, 0;
    }

    if ( $port =~ /^u\d+$/ ) {
        my ( $var, $ref ) = Lxcr_info_ctl($ctl);

        return 0, 11 unless ( $ref->{$ctl}{"Unit"}{$port}{"Status"} );
        return 0,
          raid_smart::translate_text_to_code(
            $ref->{$ctl}{"Unit"}{$port}{"Status"} );
    }

    if ( $port =~ /^p\d+$/ ) {
        my ( $var, $ref ) = Lxcr_info_ctl($ctl);

        return 0, 11 unless ( $ref->{$ctl}{"Drive"}{$port}{"Status"} );
        return 0,
          raid_smart::translate_text_to_code(
            $ref->{$ctl}{"Drive"}{$port}{"Status"} );
    }

    return 0, "Bad command";
}

## ###################################
## Fonction state :
##
## state()

sub state {
    my $ctl = shift;
    my ( $var, $ref ) = Lxcr_info_ctl($ctl);

    return 0, $ref->{$ctl}{"NotOpt"};
}

## ###################################
## Fonction capacity :

sub capacity {
    my $ctl = shift;

    my $raid_support = qx#$Lxcr_name  -C -g -z allinfo#;
    return 0, $raid_support;
}

## ###################################
## Fonction info :

sub info {
    my $ctl = shift;

    my $raid_support = qx#$Lxcr_name  -C -g -z allinfo#;
    return 0, $raid_support;
}

## ###################################
## Fonction bbu :

sub bbu {
    my $ctl = shift;
    my $bbuinfo;

    my @ctl_info = qx($Lxcr_name  -C -g -z allinfo);

    my $found_ctl;
    my $ctlregexp = qr/\*+Controller\s($ctl)/;

    foreach (@ctl_info) {
        if (m/$ctlregexp/) {
            $found_ctl++;
            next;
        }

        if ($found_ctl) {
            $bbuinfo .= $_
              if m/^Battery/;

            last if m/^Battery\sHoldup/;
        }
    }

    return 0, $bbuinfo;

}

## ###################################
## Fonctions prives
##
## raid_level

sub _raid_level {
    my $raid_level = shift;
    my $ctl        = shift;
    my $unit       = shift;

    my $span = qx#$Lxcr_name  -C -g -z allinfo#;
    chomp $span;

    if (
        $raid_level =~ /Primary-(\d+), Secondary-(\d+), RAID Level Qualifier-/ )
    {
        if ( $span == 1 ) {
            return "RAID-$1";
        } elsif ( $span == 2 ) {
            return "RAID-$1$2";
        }
    }

    return "NA";
}

# modle contrleur
sub _ctlmodel {
    my @retour_info = qx#$Lxcr_name  -C -g -z allinfo#;
    my $ctl         = shift;

    my $conttype;

    my $typeregexp = qr/^\w+\stype\sis\s(\w+)/;
    my $ctlregexp  = qr/\*+Controller\s(\d+)/;

    foreach (@retour_info) {
        if (m/$typeregexp/) {
            $conttype = $1;
        }
        if (m/$ctlregexp/) {
            my $ctlnum = $1;
            return $conttype
              if $ctl == $ctlnum;
        }
    }

    return "Unknown";
}

1;
