#!/usr/bin/perl -w
## ######### PROJECT DESCRIPTION : ###
## 
## lib fontions MegaCli
## 
## ###################################
## 
## Made by Pujos Sylvain
## 
## Started on  Mon May 28 13:23:04 2007 Pujos Sylvain
## Last update Mon May 28 13:26:45 2007 Pujos Sylvain
## 
## ###################################
##

## ###################################
## Declaration du package

package MegaCli_fct_v1;

## ###################################
## Appel des librairies

use	strict;
use Data::Dumper;
use MegaCli_info_v1;

## ###################################
## Declaration de variables

my $megacli_name	= "/usr/sbin/MegaCli";

## ###################################
## Fonction createunit :
##
## createunit($cid, $RaidType, $PidList)

sub createunit {
	my $ctl		= shift;
	my $type	= shift;
	my ($port)	= @_;

	my @liste_enclosure = qx#$megacli_name -encinfo -a $ctl -NoLog | grep "Device ID"#;
	return ( -1, "createunit ERROR : enclosure unkonwn") if ( scalar @liste_enclosure != 1 );
	my $enclosure = $liste_enclosure[0];
	$enclosure =~ s/^.*Device ID\s*:\s*(\d+).*\n/$1/;

	if ( $type eq "spare" ) {
		$port =~ s/p//;
		my $retour = qx#$megacli_name -PDHSP -set -physdrv[$enclosure:$port] -a $ctl -NoLog 2>&1#;
	
		return 1, $retour if ( $? );
		return 0, $retour if ( $? eq 0 );
		return -1, "createunit ERROR : $megacli_name -PDHSP -set -physdrv[$enclosure:$port] -a $ctl -NoLog";
	}
	if ( ( $type eq "jbod" ) or ( $type eq "single" ) ) {
		return 1, "This RAID is not supported by this card : $type";
	}
	
	my $liste_port;
	if ( $port =~ /p(\d+)-(\d+)/ ) {
		my @min = sort ($1, $2);
		for ( my $i = $min[0]; $i<=$min[1]; $i++) {
			$liste_port .= $enclosure .":". $i .",";
		}
		chop $liste_port;
	}
	elsif ( $port =~ /p(\d+:\d+.*)/ ) {
		$liste_port = $enclosure .":". join ",$enclosure:", (split ":", $1 );
	}
	else {
		foreach ( @_ ) {
			$liste_port .= $enclosure .":". $1 ."," if ( m/p(\d+)/ );
		}
		chop $liste_port;
	}

	my $raid_support = qx#$megacli_name  -AdpAllInfo -a $ctl -NoLog 2>&1 | grep "^RAID Level Supported"#;
	unless ( $raid_support =~ /$type/i ) {
		return 1, "This RAID is not supported by this card : $type";
	}

	if ( ( $type eq "raid0" ) or ( $type eq "raid1" ) or ( $type eq "raid5" ) or ( $type eq "raid6" ) ) {
		$type =~ s/raid//;

		my $ret = qx#$megacli_name -CfgLDAdd -R$type [$liste_port] -a $ctl -NoLog 2>&1#;
		if ( $? eq 0 ) {
			my $nb_ld = qx#$megacli_name -LDGetNum -a $ctl -NoLog 2>&1 | grep "Number of Virtual"#;
			if ( $nb_ld =~ m/:\s*(\d+)\n/ ) {
				my $nb = $1 - 1;
		
				my $ret2 = qx#$megacli_name -LDInit -Start Full -L$nb -a $ctl -NoLog 2>&1#;
				return 1, $ret2 if ( $? );
				return 0, $ret.$ret2 if ( $? eq 0);
				return -1, "createunit ERROR : $megacli_name -LDInit -Start Full -L$nb -a $ctl -NoLog";
			}
		}
		return 1, $ret if ( $? );
		return -1, "createunit ERROR : $megacli_name -CfgLDAdd -R$type [$liste_port] -a $ctl -NoLog";
	}
	
	if ( ( $type eq "raid10" ) or ( $type eq "raid50" ) or ( $type eq "raid60" ) ) {
		$type =~ s/raid//;

		my @tab		= split ",", $liste_port;
		my ( $array0, $array1 );
		while ( @tab ) {
			$array0 .= shift (@tab) .",";
			$array1 .= pop (@tab) ."," if @tab;
		}
		chop $array0;
		chop $array1;
		
		my $ret = qx#$megacli_name -CfgSpanAdd -R$type -Array0[$array0] -Array1[$array1] -a $ctl -NoLog 2>&1#;
		if ( $? eq 0 ) {
			my $nb_ld = qx#$megacli_name -LDGetNum -a $ctl -NoLog 2>&1 | grep "Number of Virtual"#;
			if ( $nb_ld =~ m/:\s*(\d+)\n/ ) {
				my $nb = $1 - 1;
		
				my $ret2 = qx#$megacli_name -LDInit -Start Full -L$nb -a $ctl -NoLog 2>&1#;
				return 1, $ret2 if ( $? );
				return 0, $ret.$ret2 if ( $? eq 0);
				return -1, "createunit ERROR : $megacli_name -LDInit -Start Full -L$nb -a $ctl -NoLog";
			}
		}
		return 1, $ret if ( $? );
		return -1, "createunit ERROR : $megacli_name -CfgSpanAdd -R$type -Array0[$array0] -Array1[$array1] -a $ctl -NoLog";
	}

	return 1, "This RAID is unknown : $type";
}

## ###################################
## Fonction deleteunit :
## 
## deleteunit($cid, $uid)

sub deleteunit {
	my ($ctl, $unit) = @_;
	$unit =~ s/u//;
	my $ret;

	if ( $unit =~ /^S(\d*)/ ) {
		my $port = $1;
		my @liste_enclosure = qx#$megacli_name -encinfo -a $ctl -NoLog | grep "Device ID"#;

		return ( -1, "deleteunit ERROR : enclosure unkonwn") if ( scalar @liste_enclosure != 1 );
		my $enclosure = $liste_enclosure[0];
		$enclosure =~ s/^.*Device ID\s*:\s*(\d+).*\n/$1/;
	
		$ret = qx#$megacli_name -PDHSP -Rmv -physdrv[$enclosure:$port] -a $ctl -NoLog 2>&1#;
	}
	else {
		my $state_init = qx#$megacli_name -LDInit -ShowProg -L$unit -a$ctl -NoLog 2>&1#;
		if ( $state_init =~ m/Complete/ ) {
			qx#$megacli_name -LDInit -Stop -L$unit -a$ctl -NoLog 2>&1#;
		}
		$ret = qx#$megacli_name -CfgLDDel -L$unit -a$ctl -NoLog 2>&1#;
	}

	return 1, $ret if ( $? );
	return 0, $ret if ( $? eq 0 );
	return -1, "deleteunit ERROR : $megacli_name -CfgLDDel -L$unit -a$ctl -NoLog";
}


## ###################################
## Fonction rescan :
## 
## rescan($cid)

sub rescan {
	return 1, "RESCAN Not Found with LSI";
}


## ###################################
## Fonction verify :
## 
## verify($cid, $uid)

sub verify {
	my ($ctl, $unit) = @_;
	$unit =~ s/u//;

	my $ret = qx#$megacli_name -LDCC -Start -L$unit -a$ctl -NoLog 2>&1#;
	
	return 1, $ret if ( $? );
	return 0, $ret if ( $? eq 0 );
	return -1, "verify ERROR : $megacli_name  -LDCC -Start -L$unit -a$ctl -NoLog";
}

### PAS FAIT alarms ###
## ###################################
## Fonction alarms :
## 
## alarms([$cid]) 

sub alarms {
	my $ctl = shift;
	if ( @_ ) {
		my $cmd = shift;
		
		if ( $cmd eq "clear" ) {
			qx#$megacli_name -AdpEventLog -Clear -a $ctl -NoLog#;
			return 0, "";
		}
	}
	my %nettoyage = (
		"Seconds" => {
			"0x00000000"	=> 0,
			"0x00000001"	=> 0,
			"0x00000004"	=> 0,
			"0x00000010"	=> 0,
			"0x0000001e"	=> 0,
			"0x00000026"	=> 0,
			"0x00000036"	=> 0,
			"0x0000005b"	=> 0,
			"0x0000008e"	=> 0,
			"0x000000c3"	=> 0,
			"0x000000ed"	=> 0,
			"0x000000ee"	=> 0,
			"0x000000ef"	=> 0,
			"0x000000f7"	=> 0,
			"0x00000101"	=> 0,
		},
		"Time" => {
			"0x00000004"	=> 0,
			"0x00000023"	=> 0,
			"0x00000027"	=> 0,
			"0x0000002c"	=> 0,
			"0x00000036"	=> 0,
			"0x0000003b"	=> 0,
			"0x0000003f"	=> 0,
			"0x00000040"	=> 0,
			"0x00000042"	=> 0,
			"0x00000051"	=> 0,
			"0x00000055"	=> 0,
			"0x00000056"	=> 0,
			"0x00000072"	=> 0,
			"0x00000087"	=> 0,
			"0x00000088"	=> 0,
			"0x0000008a"	=> 0,
			"0x0000008b"	=> 0,
			"0x000000da"	=> 0,
			"0x000000db"	=> 0,
			"0x000000dc"	=> 0,
			"0x000000f7"	=> 0,
			"0x000000f8"	=> 0,
		},
	);
	
	my %severity = (
		"0"	=> "INFO    ",
		"1"	=> "WARN    ",
		"2"	=> "ERROR   ",
		"3"	=> "CRITICAL",
	);
	
	my @file;
	my @tab;
	my $time	= "";
	my $code	= "";
	
	my @events = qx#$megacli_name -AdpEventLog -GetEvents -f /tmp/getevents.txt -a $ctl -NoLog; cat /tmp/getevents.txt #;
	
	foreach ( @events ) {
		next if ( m/^$/ );
	
		if ( m/^seqNum:/ ){
			push @file, [ @tab ] unless ( exists $nettoyage{ $time }{ $code } );
	
			undef @tab;
			$time	= "";
			$code	= "";
			next;
		}
	
		if ( m/^Seconds since last reboot:/ ) {
			$time	= "Seconds";
			$tab[0] = " At boot time           ";
		}
		if ( m/^Time:\s*(.*)/ ) {
			$time	= "Time";
			$tab[0] = $1;
		}
		$code	= $1 if ( m/^Code:\s*(0x\S+)/ );
	
		$tab[1]	= $1 if ( m/^Class:\s*(.*)/ );
		$tab[2]	= $1 if ( m/^Event Description:\s*(.*)/ );
		$tab[2]	= $1 if ( m/^Event Description:\s*(Unexpected sense:\s*.*), CDB/ );
	}
	
	shift @file;
	
	my $msg;
	foreach ( @file ) {
		my @tab = @{ $_ };
	
		$msg .= " $ctl   " .$tab[0]. "    ";
		if ( exists $severity{ $tab[1] } ) {
			$msg .= $severity{ $tab[1] };
		}
		else {
			$msg .= "        ";
		}
		
		$msg .= "  ". $tab[2] ." \n";
	}
	
#	return 1, $msg if ( $? );
	return 0, $msg;
	# if ( $? eq 0 );
#	return -1, "alarms ERROR : $megacli_name alarms $arg";
}

## ###################################
## Fonction info :
## 
## info()

sub info {
	my $arg = join (",", @_);
	
	my $ret = qx#$megacli_name -adpAllInfo -a$arg -NoLog 2>&1#;
	
	return 1, $ret if ( $? );
	return 0, $ret if ( $? eq 0 );
	return -1, "info ERROR : $megacli_name info $arg";
}

## ###################################
## Fonction rebuilt :
## 
## rebuild($cid, $uid, $pid)

sub rebuild {
	my ($ctl, $unit, $port) = @_;
	$unit =~ s/u//;
	$port =~ s/p//;

	my $liste_unit = qx#$megacli_name -LDInfo -L $unit -a $ctl -NoLog | grep '^RAID Level'#;
	my $raid = MegaCli_info_v1::_raid_level( $liste_unit, $ctl, $unit );
	
	return ( -1, "rebuild ERROR : raid_level unkonwn") if ( $raid eq 'NA' );
	$raid =~ s/RAID-//;

	my @liste_enclosure = qx#$megacli_name -encinfo -a $ctl -NoLog | grep "Device ID"#;
	
	return ( -1, "rebuild ERROR : enclosure unkonwn") if ( scalar @liste_enclosure != 1 );
	my $enclosure = $liste_enclosure[0];
	$enclosure =~ s/^.*Device ID\s*:\s*(\d+).*\n/$1/;

	my $ret = qx#$megacli_name -LDRecon -Start  -r$raid -Add -PhysDrv[$enclosure:$port] -L$unit -a$ctl -NoLog 2>&1#;

	return 1, $ret if ( $? );
	return 0, $ret if ( $? eq 0 );
	return -1, "rebuild ERROR : $megacli_name -LDRecon -Start  -r$raid -Add -PhysDrv[$enclosure:$port] -L$unit -a$ctl -NoLog";
}


## ###################################
## Fonction deleteport :
## 
## deleteport($cid, $pid)

sub deleteport {
	return 1, "DELETEPORT Not Found with LSI";
}

1;
