#############################################################
##
## Made by PUJOS Sylvain
##
## Started on  Mon May 28 11:14:21 2007 PUJOS Sylvain
## Last update Mon May 28 17:17:02 2007 Pujos Sylvain
##

# nom du package:
package MegaCli_info_v1;

use strict;
use Data::Dumper;
use raid_smart;

my $megacli_name	= "/usr/sbin/MegaCli";


##################################
#recuperation des infos sur la ou les carte(s) presente(s)
#le hash %megacli est retourne
sub megacli_info_ctl {
	my $ctl = shift;
	# dfaut :tous les contrleurs ?
	# $ctl |= "All";
	
	my %megacli;
	my %resultat_ctl;
	my @line_ctl = qx#$megacli_name -adpAllInfo -a $ctl -NoLog#;
	return 0, undef unless ( @line_ctl );

	# Recuperation des infos
	foreach ( @line_ctl ) {
		m/Adapter\s\#(\d+)/					and $ctl = $1						and next ;
		m/Product Name\s+:\s+([\w\s\d]+)/	and $resultat_ctl{'Model'}	= $1	and chomp $resultat_ctl{'Model'} and next ;
		m/Number of Backend Port:\s+(\d+)/	and $resultat_ctl{'Ports'}	= $1	and next ;
		m/^\s+Disks\s+:\s+(\d+)/			and $resultat_ctl{'Drives'}	= $1	and next ;
		m/^Virtual Drives\s+:\s+(\d+)/		and $resultat_ctl{'Units'}	= $1	and next ;
		m/^\s+Critical Disks\s+:\s+(\d+)/	and $resultat_ctl{'NotOpt'}	+= $1	and next ;
		m/^\s+Failed Disks\s+:\s+(\d+)/		and $resultat_ctl{'NotOpt'}	+= $1	and next ;
		m/^\s+Degraded\s+:\s+(\d+)/			and $resultat_ctl{'NotOpt'}	+= $1	and next ;
	}  
	
	$megacli{$ctl} = \%resultat_ctl;

	# Recuperation des infos sur chaque unite
	my @mini_liste_unit = qx#$megacli_name -LDInfo -Lall -a $ctl -NoLog | grep "Virtual Disk"#;

	foreach (@mini_liste_unit) {
		if ( m/^Virtual Disk:\s+(\d+)/ ){
			my $unit = $1;
			my %resultat_unit;

			my @liste_unit = qx#$megacli_name -LDInfo -L $unit -a $ctl -NoLog#;
			foreach my $line_unit (@liste_unit) {
				
				if ( $line_unit =~ m/^RAID Level:\s*(.*)$/ ) {
					$resultat_unit{'UnitType'}	= _raid_level( $1, $ctl, $unit);
					next;
				}

				if ( $line_unit =~ m/^State:\s*(\w+)/ ) {
					my $test = $1;
					$resultat_unit{'Status'}	= $test;
					$resultat_unit{'Status'}	= "OK"			if ( $test =~ m/^Optimal$/ );
					$resultat_unit{'Status'}	= "DEGRADED"	if ( $test =~ m/^Degraded$/ );
					next ;
				}

				if ( $line_unit =~ m/^Size:\s*(\w+)/ ) {
					if ( $1 =~ /^(\d*)KB$/ ) {
						$resultat_unit{'Size'}		= $1 * 1024 / 1000 / 1000 / 1000;
						$resultat_unit{'Blocks'}	= $1 * 1024;
					}
					elsif ( $1 =~ /^(\d*)MB$/ ) {
						$resultat_unit{'Size'}		= $1 / 1000;
						$resultat_unit{'Blocks'}	= $1 * 1024 * 1024;
					}
					elsif ( $1 =~ /^(\d*)GB$/ ) {
						$resultat_unit{'Size'}		= $1 * 1024 * 1024 * 1024 / 1000 / 1000 / 1000;
						$resultat_unit{'Blocks'}	= $1 * 1024 * 1024 * 1024;
					}
					elsif ( $1 =~ /^(\d*)TB$/ ) {
						$resultat_unit{'Size'}		= $1 * 1024 * 1024 * 1024 * 1024 / 1000 / 1000 / 1000;
						$resultat_unit{'Blocks'}	= $1 * 1024 * 1024 * 1024 * 1024;
					}
					else {
						$resultat_unit{'Size'}		= "NA";
						$resultat_unit{'Blocks'}	= "NA";
					}
					next;
				}
				
				if ( $line_unit =~ m/^\s+Reconstruction\s*:\s*Completed\s*(\d+)%/ ) {
					$resultat_unit{'Cmpl'}		= $1;
					$resultat_unit{'Status'}	= "REBUILDING";
					$megacli{$ctl}{'NotOpt'}	+= 1;
				}
				elsif ( $line_unit =~ m/\s+Initialization\s*:\s*Completed\s*(\d+)%/ ) {
					$resultat_unit{'Cmpl'}		= $1;
					$resultat_unit{'Status'}	= "INITIALIZING";
					$megacli{$ctl}{'NotOpt'}	+= 1;
				}
				elsif ( $line_unit =~ m/\s+Check Consistency\s*:\s*Completed\s*(\d+)%/ ) {
					$resultat_unit{'Cmpl'}		= $1;
					$resultat_unit{'Status'}	= "VERIFYING";
					$megacli{$ctl}{'NotOpt'}	+= 1;
				}
				else {
					$resultat_unit{'Cmpl'} = "-";
				}
			}
			
			$megacli{$ctl}{'Unit'}{"u".$unit} = \%resultat_unit;
		}
	}
	
	# Recuperation des ports sur chaque unite
	my @liste_unit_port = qx#$megacli_name -LDPDInfo -a $ctl -NoLog | grep -E '^Slot Number|^Virtual'#;
	my %unit_port;
	my $vd;
	foreach my $line (@liste_unit_port) {
		if ( $line =~ m/^Virtual Disk:\s*(\d+)/ ){
			$vd = $1;
			next;
		}
		if ( $line =~ m/^Slot Number:\s*(\d+)/ ) {
			$unit_port{ "p".$1 } = "u".$vd;
		}
	}
	
	# Recuperation des infos sur chaque controleur : disques
	my @mini_liste_port = qx#$megacli_name -PDList -a $ctl -NoLog | grep -E "^Slot Number|^Enclosure Device"#;
	my $enclosure;

	foreach (@mini_liste_port) {
		if ( m/^Enclosure Device ID:\s+(\d+)/ ){
			$enclosure = $1;
			next;
		}
		if ( m/^Slot Number:\s+(\d+)/ ){
			my $port_id	= $1;
			my $port	= "p".$port_id;
			my %resultat_port;
			my $model;

			my @liste_port = qx#$megacli_name -PDInfo -PhysDrv[$enclosure:$port_id] -a $ctl -NoLog#;
			foreach my $liste_port (@liste_port) {
				
				if ( $liste_port =~ m/^Firmware state:\s*(.+)/ ) {
					my $test = $1;
					$resultat_port{'Status'}	= $test;
					$resultat_port{'Status'}	= "OK" if ( $test =~ m/^Online$/ );
					$resultat_port{'Status'}	= "OK" if ( $test =~ m/^Unconfigured\(good\)$/ );
					
					if ( $test =~ m/^Rebuild$/ ) {
						$resultat_port{'Status'}	= "REBUILDING";
						my $cmpl = qx#$megacli_name -PDRbld -ShowProg -PhysDrv[$enclosure:$port_id] -a $ctl -NoLog#;
						if ( $cmpl =~ m/Completed\s*(\d+)%/ ) {
							$megacli{$ctl}{'Unit'}{ $unit_port{ $port } }{'Cmpl'} = $1;
						}
					}
					if ( $test =~ m/^Hotspare$/ ) {
						$resultat_port{'Status'}	= "OK";
						$unit_port{ $port } = "uS".$port_id;

						if ( not exists $megacli{$ctl}{'Unit'}{"uS".$port_id} ) {
							my %resultat_unit;
							$resultat_unit{'UnitType'}	= "SPARE";
							$resultat_unit{'Status'}	= "OK";
							$resultat_unit{'Size'}		= "";
							$megacli{$ctl}{'Unit'}{"uS".$port_id} = \%resultat_unit;
						}
					}
					next ;
				}
				if ( $liste_port =~ m/^Non Coerced Size:\s*(\w+)\s+/ ) {
					if ( $1 =~ /^(\d*)KB$/ ) {
						$resultat_port{'Size'}		= $1 / 1024 / 1024;
						$resultat_port{'Blocks'}	= $1 * 1024;
					}
					elsif ( $1 =~ /^(\d*)MB$/ ) {
						$resultat_port{'Size'}		= $1 / 1024;
						$resultat_port{'Blocks'}	= $1 * 1024 * 1024;
					}
					elsif ( $1 =~ /^(\d*)GB$/ ) {
						$resultat_port{'Size'}		= $1 ;
						$resultat_port{'Blocks'}	= $1 * 1024 * 1024 * 1024;
					}
					elsif ( $1 =~ /^(\d*)TB$/ ) {
						$resultat_port{'Size'}		= $1 * 1024;
						$resultat_port{'Blocks'}	= $1 * 1024 * 1024 * 1024 * 1024;
					}
					else {
						$resultat_port{'Size'}		= "NA";
						$resultat_port{'Blocks'}	= "NA";
					}
					next;
				}
				
				if ( $liste_port =~ m/^Inquiry Data:\s+\w+\s+(\w+)\s+(\w+)/ ){
					$resultat_port{'Serial'}	= $2;
					$model						= $1;
				}
			}
			
			$resultat_port{'Unit'} = "-";
			$resultat_port{'Unit'} = $unit_port{ $port } if ( exists $unit_port{ $port } );
			
			$megacli{$ctl}{'Drive'}{$port} = \%resultat_port;
			$megacli{$ctl}{'Drive'}{$port}{'Model'} = $model;
			push @{$megacli{$ctl}{'Unit'}{ $resultat_port{'Unit'} }{'Port'}}, $port;
		}
	}

	return 0, \%megacli;
}

##################################
#recuperation de la liste des controleurs presentes
sub megacli_list_ctl {
	my @retour_info = qx#$megacli_name -adpCount -NoLog#;
	my @list_ctl;
	
	my $contrcount;
	 
	foreach ( @retour_info ) {
		( $contrcount ) = ( m/Controller Count:\s+(\d+)/ ) ;
	}
	
	for ( 1 .. $contrcount ) {
		# get all info for controller
		my $ctlnum = $_ -1;
		my $line_ctl = qx#$megacli_name -adpAllInfo -a $ctlnum -NoLog | grep "Product Name" #;
		
		( $list_ctl[$ctlnum] ) = ( $line_ctl =~ m/Product Name\s+:\s+([\w\s\d]+)/ )  ;
		chop $list_ctl[$ctlnum] ;
		$list_ctl[$ctlnum] =~ s/\s+/_/g ;
		$list_ctl[$ctlnum] .= "_$ctlnum";
	}

	return 0, \@list_ctl;
}

##################################
#recuperation de la liste des unites pour un controleur donne
sub megacli_list_unit {
	my $ctl = shift;

	my @liste_unit_port = qx#$megacli_name -LDPDInfo -a $ctl -NoLog | grep '^Virtual'#;
	my @liste_unit;

	foreach ( @liste_unit_port ) {
		push @liste_unit, "u".$1 if ( $_ =~ m/^Virtual Disk:\s*(\d+)/ );
	}

 	return 0, \@liste_unit;
}

##################################
#recuperation de la liste des ports pour un controleur donne
sub megacli_list_port {
	my $ctl = shift;

	my @liste_unit_port = qx#$megacli_name -PDList -a $ctl -NoLog | grep "^Slot Number"#;
	my @liste_port;

	foreach ( sort @liste_unit_port ) {
		push @liste_port, "p".$1 if ( $_ =~ m/^Slot Number:\s*(\d+)/ );
	}

 	return 0, \@liste_port;
}


##################################
#recuperation des infos smart pour un controleur et un port donne
sub megacli_smart {
    my ( $ctl, $port ) = @_;

    if ( ! $port ) {
		my ($var, $ref) = megacli_info_ctl( $ctl );
		return 0, 41 if ( $ref->{$ctl}{"NotOpt"} );
		return 0, 0;
    }
    
    if ($port =~ /^u\d+$/) {
		my ($var, $ref) = megacli_info_ctl( $ctl );

		return 0, 11 unless ( $ref->{$ctl}{"Unit"}{$port}{"Status"} );
		return 0, raid_smart::translate_text_to_code( $ref->{$ctl}{"Unit"}{$port}{"Status"} );
    }


    if ($port =~ /^p\d+$/) {
		my ($var, $ref) = megacli_info_ctl( $ctl );

		return 0, 11 unless ( $ref->{$ctl}{"Drive"}{$port}{"Status"} );
		return 0, raid_smart::translate_text_to_code( $ref->{$ctl}{"Drive"}{$port}{"Status"} );
    }

    return 0, "Bad command";
}

## ###################################
## Fonction state :
## 
## state()

sub state {
    my $ctl = shift;
    my ($var, $ref) = megacli_info_ctl( $ctl );

    return 0, $ref->{$ctl}{"NotOpt"};
}

## ###################################
## Fonction capacity :

sub capacity {
    my $ctl = shift;

	my $raid_support = qx#$megacli_name  -AdpAllInfo -a $ctl -NoLog 2>&1 | grep "^RAID Level Supported"#;
    return 0, $raid_support;
}

## ###################################
## Fonction private :
## 
## raid_level

sub _raid_level {
	my $raid_level	= shift;
	my $ctl			= shift;
	my $unit		= shift;
	
	my $span = qx#$megacli_name -LDInfo -L $unit -a $ctl -NoLog | grep "^Span Depth" | cut -d ":" -f 2 #;
	chomp $span;

	if ( $raid_level =~ /Primary-(\d+), Secondary-(\d+), RAID Level Qualifier-/ ) {
		if ( $span == 1 ) {
			return "RAID-$1";
		}
		elsif ( $span == 2 ) {
			return "RAID-$1$2";
		}
	}
	
	return "NA";
}

1;
