## ######### PROJECT NAME : ##########
## 
## raid_logiciel.pm for  in /home/dayan/intellique/raid
## 
## ######### PROJECT DESCRIPTION : ###
## 
## raid logiciel
## 
## ###################################
## 
## Made by Dayan Romain
## Login   <dayan@intellique.com>
## 
## Started on  Tue Jul 19 09:14:35 2005 Dayan Romain
## Last update Wed May 16 17:27:39 2007 Boutonnet Alexandre
## 
## ###################################
##

package raid_logiciel;

use strict;
use raid_logiciel_v1;
use raid_logiciel_v2;
use lib_raid;

use Data::Dumper;

sub switch {
    # Aiguille en fonction de la version et de la fonction

    # Test de la presence des parametres
    return error( "Not enough argument" ) if ( $#_ < 0 );

    #####################################
    # Declaration de variables

    # Variable contenant les references des fonctions selon la version
    my %possibility = (
        "raid_logiciel_v1"	=> {
            info		=> \&raid_logiciel_v1::info,
            rescan  	=> \&raid_logiciel_v1::rescan,
            deleteport  => \&raid_logiciel_v1::deleteport,
            deleteunit  => \&raid_logiciel_v1::deleteunit,
            createunit  => \&raid_logiciel_v1::createunit,
            rebuild 	=> \&raid_logiciel_v1::rebuild,
            verify	    => \&raid_logiciel_v1::verify,
            alarms	    => \&raid_logiciel_v1::alarms,
            state	    => \&raid_logiciel_v1::state,
            list_unit	=> \&raid_logiciel_v1::list_unit,
            list_port	=> \&raid_logiciel_v1::list_port,
            smart	    => \&raid_logiciel_v1::smart
        },
        "raid_logiciel_v2"	=> {
            info		=> \&raid_logiciel_v2::info,
            rescan      => \&raid_logiciel_v2::rescan,
            deleteport	=> \&raid_logiciel_v2::deleteport,
            deleteunit	=> \&raid_logiciel_v2::deleteunit,
            createunit	=> \&raid_logiciel_v2::createunit,
            rebuild     => \&raid_logiciel_v2::rebuild,
            verify      => \&raid_logiciel_v2::verify,
            alarms      => \&raid_logiciel_v2::alarms,
            state       => \&raid_logiciel_v2::state,
            list_unit	=> \&raid_logiciel_v2::list_unit,
            list_port	=> \&raid_logiciel_v2::list_port,
            smart       => \&raid_logiciel_v2::smart,
            bbu         => \&raid_logiciel_v2::bbu,
            cache       => \&raid_logiciel_v2::cache,
            capacity    => \&raid_logiciel_v2::capacity,
        }
    );


    #####################################
    # Initialisation des parametres
    my $version		= version();
    my $function	= shift;

    #####################################
    # Processus d'aiguillage
    return error( "Function unknown" ) unless ( $possibility{ $version }{ $function } );
    return $possibility{ $version }{ $function }->( @_ );
}

sub info_all {
    # Un parametre est fourni
    my $name = shift;

    my %retour;
    my ($retour_clo, $clo) = switch("info", $name);

    if ( $retour_clo == 1 ) {
		return error( $clo );
    }

    foreach ( keys %{ $clo }) {
	$retour{$_} = $clo->{$_};
    }

    return ok( \%retour );
}

sub list_ctl {
    # Un parametre est fourni
    if ( mdadm_exist() ) {
		return ok( \@_ );
    }

    return error( undef );
}

sub alarms {
    # Un parametre est fourni
	my ($retour_clo, $clo) = switch("alarms", @_);

    if ( $retour_clo == 1 ) {
		return error( $clo );
    }
    return ok( $clo );
}

sub state {
    # Un parametre est fourni
    my ($retour_clo, $clo) = switch("state", $_[0]);

    return error( $clo ) if ( $retour_clo == 1 );
    return ok( $clo );
}

sub version {
    # Renvoie la version du raid logiciel

    my $version = qx|/sbin/mdadm --version 2>&1|;
    
    return "raid_logiciel_v1" if ($version =~ m/.*1\.9\.0.*/);
    return "raid_logiciel_v2" if ($version =~ m/.*2\.5\.6.*/);
	return "raid_logiciel_v2" if ($version =~ m/.*3\.\d\.\d.*/);

    return "unknown";
}

sub mdadm_exist {
    return 0 unless ( -e "/sbin/mdadm" );
    return 0 unless ( -x "/sbin/mdadm" );
    return 1;
}

1;
