# --- fichier raid_logiciel_info_v2.pm ---
# Information pour le raid logiciel v2
#############################################################
##
## Made by Boutonnet Alexandre based on DAYAN's ugly code.
##
## Started on  Mon July 20 11:14:21 CET 2005 
## Last update Wed May 16 16:32:29 2007 Boutonnet Alexandre
##

# nom du package:
package raid_logiciel_info_v2;

use strict;
use lib_raid;
use raid_smart;

##################################

my $mdadm = "/sbin/mdadm";

my %traduction = (
    "active"                        => "OK",
    "active sync"                   => "OK",
    "clean"                         => "OK",
    "spare rebuilding"              => "REBUILDING",
    "clean, resyncing"              => "REBUILDING",
    "clean, degraded"               => "DEGRADED",
    "clean, degraded, recovering"   => "DEGRADED",
    "active, resyncing"             => "REBUILDING",
    "active, degraded, recovering"  => "REBUILDING",
    "active, degraded"              => "DEGRADED",
    "faulty"                        => "UNKNOWN",
    "raid0"                         => "RAID-0",
    "raid1"                         => "RAID-1",
    "raid2"                         => "RAID-2",
    "raid3"                         => "RAID-3",
    "raid4"                         => "RAID-4",
    "raid5"                         => "RAID-5",
    "raid6"                         => "RAID-6",
);

##################################
#recuperation des infos sur le controleur logiciel
#le hash %rl est retourne
sub info {
    my $ctl = shift;

    my ($err, $ret) = _test_mdadm_binary();
    return $err, $ret if $err;

    my @retour_info = `$mdadm --detail --scan -v`;
    chomp @retour_info;

    return 0, undef unless ( @retour_info );

    my %rl;
    $rl{$ctl}{'Drives'} = 0;
    $rl{$ctl}{'Units'}  = 0;
    $rl{$ctl}{'NotOpt'} = 0;
    $rl{$ctl}{'Ports'}  = '___';
    $rl{$ctl}{'Model'}  = '___';

    # Recuperation des infos comme-ci dessous
    # ???    Unit    UnitType    Ss_Nb_device    ???    Port
    while (@retour_info) {
        my $line_array = shift @retour_info;
        if ( $#retour_info < 0 ) {
            $line_array .= "   devices=,";
        } else {
            $line_array .= shift @retour_info;
        }
    
        my %resultat_unit;
        my $unit;
        my $unit_type;
        my $ss_nb_devices;
        my $list_ports;
    
        ($unit,
         $unit_type,
         $ss_nb_devices,
         $list_ports) = ($line_array =~ m/ARRAY\s*(\S*)\s*level=(\S*)\s*num-devices=(\S*)\s*.*\s*devices=(\S*)/);
    
        $resultat_unit{'Port'} = [ split ",", $list_ports ];
        $resultat_unit{'UnitType'} = $traduction{ $unit_type };
    
        $rl{$ctl}{'Drives'} += $ss_nb_devices;
        $rl{$ctl}{'Units'} ++;
        $rl{$ctl}{'Unit'}{$unit} = \%resultat_unit;
    
        $rl{$ctl}{'Unit'}{$unit}{'Cmpl'} = '-';
    
    
        # Recuperation des infos sur chaque unite comme-ci dessous
        my @retour_info_unit = `mdadm --detail $unit`;
        chomp @retour_info_unit;
    
        foreach my $line_unit (@retour_info_unit) {
            if ( $line_unit =~ m/Array Size :\s(\S*)\s+\((\S*)\s/i ) {
                $rl{$ctl}{'Unit'}{$unit}{'Size'} = octet_to_giga( $1 * 1024 );
                $rl{$ctl}{'Unit'}{$unit}{'Blocks'} = $1;
                next;
            }
            if ( $line_unit =~ m/State :\s(.*)/i ) {
                $rl{$ctl}{'NotOpt'} = 1 unless ( $traduction{ $1 } eq "OK" );
                $rl{$ctl}{'Unit'}{$unit}{'Status'} = $traduction{ $1 };
                next;
            }
            if ( $line_unit =~ m/Rebuild Status :\s(.*%)/i ) {
                $rl{$ctl}{'Unit'}{$unit}{'Cmpl'} = $1;
                next;
            }
            if ( $line_unit =~ m/:/i ) {
                next;
            }
            unless ( $line_unit =~ m/(\S+\s?\S+)\s+(\/dev\/.*)$/i ) {
                next;
            }
    
            $rl{$ctl}{'Drive'}{$2}{'Status'}    = $traduction{ $1 };
            $rl{$ctl}{'Drive'}{$2}{'Unit'}        = $unit;
            $rl{$ctl}{'Drive'}{$2}{'Serial'}    = '___';
            $rl{$ctl}{'Drive'}{$2}{'Size'}        = '-';
            $rl{$ctl}{'Drive'}{$2}{'Blocks'}    = '-';
    
            my $port = $2;
            my $disc = $2;
            $disc =~ s/\/dev\/(\Sd\S).*/$1/;
    
            if ( $disc =~ m/hd/ ) {
                chomp( $rl{$ctl}{'Drive'}{$port}{'Model'} = `cat /proc/ide/$disc/model` );
            } elsif ( $disc =~ m/sd/ ) {
                chomp( my $model = `scsi_info /dev/$disc | grep MODEL | cut -d '"' -f 2` );
                $model =~ s/^ATA\s(.*)$/$1/;
                $rl{$ctl}{'Drive'}{$port}{'Model'} = $model;
            } else {
                $rl{$ctl}{'Drive'}{$port}{'Model'} = "";
            }
    
            if ( `fdisk -l $port 2> /dev/null | grep $port` =~ m/\s(\d+)\s+\w+$/) {
                $rl{$ctl}{'Drive'}{$port}{'Size'} = octet_to_giga( $1 );
                $rl{$ctl}{'Drive'}{$port}{'Blocks'} = $1 / 512;
            }
        }
    }

    return 0, \%rl;
}

##################################
#recuperation de la liste des unites sur le controleur logiciel
#le hash %rl est retourne
sub list_unit {

    my ($err, $ret) = _test_mdadm_binary();
    return $err, $ret if $err;

    my @retour_info = `$mdadm --detail --scan`;
    chomp @retour_info;

    return 0, undef unless ( @retour_info );

    my @list_unit;

    # Recuperation des infos comme-ci dessous
    # ???    Unit    UnitType
    while (@retour_info) {
        my $line_array = shift @retour_info;

        push @list_unit, $1 if ( $line_array =~ m/ARRAY\s*(\S*)\s*level/ );
    }

    return 0, \@list_unit;
}

##################################
#recuperation de la liste des ports sur le controleur logiciel
#le hash %rl est retourne
sub list_port {

    my ($err, $ret) = _test_mdadm_binary();
    return $err, $ret if $err;

    my @retour_info = `$mdadm --detail --scan`;
    chomp @retour_info;

    return 0, undef unless ( @retour_info );

    my @list_port;

    # Recuperation des infos comme-ci dessous
    # ???    Unit    UnitType    Ss_Nb_device    ???    Port
    while (@retour_info) {
        my $line_array = shift @retour_info;
        if ( $line_array =~ m/\sdevices=(\S*)/ ) {
            push @list_port, split ",", $1;
        }
    }

    return 0, \@list_port;
}

##################################
#recuperation des infos smart sur le disque
sub smart {
    my ( $ctl, $port ) = @_;

    if ( ! $port ) {
        my ( $err, $ref_info ) = info( "clo" );
        return $err, $ref_info if $err;

        return 1, "No logical unit found\n" unless ( defined $ref_info );
        return 1, "No state found for clo\n" unless ( defined $ref_info->{'clo'}{'NotOpt'} );

        return 0, 0 if ( $ref_info->{'clo'}{'NotOpt'} eq "0") ;
        return 0, 13;
    }

    if ($port =~ /^\/dev\/md\d+$/) {
        my ($err, $ret) = _test_mdadm_binary();
        return $err, $ret if $err;

        my $retour_info = `$mdadm --detail $port | grep "State :"`;
        chomp $retour_info;

        return 0, 11 unless ( $retour_info );

        if ( $retour_info =~ m/State :\s(.*)/i ) {
            return 0, raid_smart::translate_text_to_code( $traduction{ $1 } );
        } else {
            return 0, raid_smart::translate_text_to_code( "UNKNOWN" );
        }
    }

    return 1, "Can't get smart info: @_";
}

sub octet_to_giga {
    my $bytes = shift;
    $bytes = int (1000 * $bytes / 1073741824) / 1000;

    return $bytes;
}

sub _test_mdadm_binary {
    return 1, "/sbin/mdadm does not exist"              unless ( -e $mdadm );
    return 1, "Can't execute /sbin/mdadm binary file"   unless ( -x $mdadm );
    return 0;
}

1;
