
## ######### PROJECT NAME : ##########
## 
## raid_logiciel_v2.pm for INTELLIQUE
## 
## ######### PROJECT DESCRIPTION : ###
## 
## raid logiciel
## 
## ###################################
## 
## Made by Dayan Romain
## Login   <dayan@intellique.com>
## 
## Started on  Tue Jul 19 09:14:35 2005 Dayan Romain
## Last update Tue May 29 11:51:12 2007 Boutonnet Alexandre
## 
## ###################################
##

package raid_logiciel_v2;

use strict;
use raid_logiciel_info_v2;
use lib_raid;

# Hash contenant l'equivalence nom de raid valeur
my %raid = (
    "raid6"     => 6,
    "raid5"     => 5,
    "raid0"     => 0,
    "raid1"     => 1,
    "spare"     => "spare",
);

# Hash contenant le nombre de device necessaire / type de raid
my %device_raid = (
    "raid6"     => 4,
    "raid5"     => 3,
    "raid0"     => 2,
    "raid1"     => 2,
    "spare"     => 1,
);


#########################################

# Outils de debug et d'affichage

#########################################

sub affiche {
    # Creer au bon format le message de retour

    my $level = shift;

    my $param = join "\n", @_;
    my @etat = ($level, $param);
    return \@etat;
}

#########################################

# Fonctions gerer par le raid logiciel

#########################################

sub alarms {
    return ok (  "" );
}

sub state {
    my $ctl = shift;
    my ($etat, $ref) = raid_logiciel_info_v2::info($ctl);

    return $etat, $ref->{$ctl}{"NotOpt"};
}

sub save_conf {
    # Sauvegarde dans le fichier mdadm.conf de la nouvelle conf

    open(FD,">/etc/mdadm/mdadm.conf") or return error($!);
    my @retour_info = `mdadm --detail --scan -v`;
    chomp @retour_info;
    return error("") unless (@retour_info);
    while (@retour_info) {
	my $line_array = shift @retour_info;
	$line_array .= shift @retour_info;

	my $unit;
	my $list_ports;

	($unit,
	 $list_ports) = ($line_array =~ m/ARRAY\s*(\S*)\s*level=\S*\s*num-devices=\S*\s*.*\s*devices=(\S*)/);

	my $dev = $list_ports;
	$dev =~ s/,/ /g;
	print FD "DEVICE  $dev\n";
	print FD "ARRAY $unit devices=$list_ports\n";
    }

    close FD;
}

sub createunit {
    #clo,  Raid[1, 0, 5, spare], partitions);

    my $ctl = shift;
    my ($raidtype, @part) = @_;
    my $nb_part = @part;

    # Analyse du type de raid souhaite
    return error( "$raidtype isn't a valid RAID type." )  unless ( exists ($raid{$raidtype}));

    # Analyse du nombre de device recu par rapport au type de raid souhaite
    return error ( "$device_raid{$raidtype} devices are needed by $raidtype." ) if ($nb_part < $device_raid{$raidtype});

    # Recherche de le partition virtuelle de destination
    my $compt;
    for ($compt = 0; $compt < 21; $compt++) {
	last unless (`mdadm --detail /dev/md$compt 2>/dev/null`);
    }

    # Verification des devices (deja prise en raid?)
    foreach (@part) {
	my $ret = `mdadm --examine $_ 2>&1`;
	return error( "$_ is already in a RAID." )  unless($?);

	my $ret2 = `/bin/mount | grep $_ 2>&1`;
	return error( "$_ is mounted" )  unless($?);
    }

    # Si l'on recoit du raid0 on le transforme en stripe
    $raidtype = "stripe" if ($raidtype eq "raid0");

    # Essai de la commande avec mdadm 
    my $part_in_line = join(" ",@part);
    my $ret_mdadm = `echo n | mdadm --create /dev/md$compt --level=$raidtype --raid-devices=$nb_part $part_in_line 2>&1`;

    # Si $? n'est pas a 0 c'est quil c produit une erreur
    return error( $ret_mdadm ) if ($?);

    # Sauvegarde dans le fichier mdadm.conf de la nouvelle conf
    save_conf();

    return ok( $ret_mdadm );
}

sub deleteport {
    # clo $port

    my $ctl = shift;
    my $port = shift;

    # trouver le device associe au port
    my ($retour_clo, $clo) = raid_logiciel_v2::info( $ctl );
    return error( $clo ) if ( $retour_clo == 1 );

    #    return error("$port non associe a une unite")  unless (exists($clo->{ $ctl }{Drive}{$port}{Unit}));
    my $ret;

    if (exists($clo->{ $ctl }{Drive}{$port}{Unit})) {
	my $dev = $clo->{ $ctl }{Drive}{$port}{Unit};
	$ret = `mdadm $dev --manage --fail $port 2>&1`;
	sleep 3;
	$ret = `mdadm $dev --manage --remove $port 2>&1`;
    }

    $ret = `mdadm --zero-superblock $port 2>&1`;
    # Si $? n'est pas a 0 c'est quil c produit une erreur
    return error( $ret ) if ($?);

    # Sauvegarde dans le fichier mdadm.conf de la nouvelle conf
    save_conf();

    return ok( "$port is successfully deleted." );
}

sub deleteunit {
    # clo $uid

    my $ctl = shift;
    my $uid = shift;

    my ($retour_clo, $clo) = raid_logiciel_v2::info( $ctl );
    if ( $retour_clo == 1 ) {
	return error( $clo );
    }

    # Arret du device
    my $stop = `mdadm --stop $uid 2>&1`;

    # Si ca c mal passe on retourne lerreur
    return error( $stop ) if ($?);

    if ( exists $clo->{ $ctl }{Unit}{ $uid }{Port} ) {

	foreach ( @{ $clo->{ $ctl }{Unit}{ $uid }{Port} } ) {
	    `mdadm --zero-superblock $_`;
	}
    }

    # Sauvegarde dans le fichier mdadm.conf de la nouvelle conf
    save_conf();

    return ok( $stop );
}

sub info {

    # Test de la presence des parametres
    if ( $#_ < 1 ) {
	return raid_logiciel_info_v2::info( @_ );
    }

    if ( $#_ == 2 and $_[2] eq "status" ) {
	my ($retour_clo, $clo) = raid_logiciel_info_v2::info( $_[0] );

	my $drive = $clo->{ $_[0] }{ Drive }{ $_[1] }{ Status };
	my $unit  = $clo->{ $_[0] }{ Unit }{ $_[1] }{ Status };

	return ok( $drive ) if ( $drive );
	return ok( $unit ) if ( $unit );
	return error( "Unit not found" );
    }

    my $retour = `mdadm --detail $_[1] 2>&1`;
    # Erreur
    if ( $? ) {
	my $retour2 = `mdadm --examine $_[1] 2>&1`;
	return error( "Unit not found" ) if ( $? );
	return ok( $retour2 );
    }

    return ok( $retour );
}

sub rescan {
    return ok( "" );
}

sub rebuild {
    my $retour = `mdadm $_[1] --manage --add $_[2] 2>&1`;
    # Erreur
    return error( $retour ) if ( $? );
    return ok( "$_[2] port is successfully added to $_[1] unit" );
}

sub verify {
    return ok( "" );
}

sub list_unit {
    return raid_logiciel_info_v2::list_unit();
}

sub list_port {
    return raid_logiciel_info_v2::list_port();
}

sub smart {
    return raid_logiciel_info_v2::smart(@_);
}

sub bbu {
    return 0, "BBU: No bbu with clo";
}

sub cache {
    return 0, "Cache: No cache with clo";
}

sub capacity {
    my @ret = sort keys %raid;
    return 0, \@ret;
}

1;
