#!/usr/bin/perl -w
## ###################################
## 
## Made by Pujos et Sardinha
## 
## ###################################
##

## ###################################
## Declaration du package

package raid_lvm_fct_v1;

## ###################################
## Appel des librairies

use	strict;
use Data::Dumper;

## ###################################
## Declaration de variables

my $pvcreate	= _test_lvm_binary( 'pvcreate' );
my $vgcreate	= _test_lvm_binary( 'vgcreate' );
my $lvcreate	= _test_lvm_binary( 'lvcreate' );
my $vgdisplay	= _test_lvm_binary( 'vgdisplay' );
my $lvremove	= _test_lvm_binary( 'lvremove' );
my $vgremove	= _test_lvm_binary( 'vgremove' );

## ###################################
## Fonction createunit :
##
## createunit($cid, $RaidType, $PidList)

sub createunit {
    my ($ctl, $raid, @arg)	= @_;

	return 1, "Createunit Not Operationnel for lvm with raid type: $raid" unless ( $raid eq "raid0" );
	
	my $params = join " ", @arg;
	my $ret = qx#$pvcreate $params 2>&1#;
    
	if ($? ne 0) {
	    return 1, $ret if ($?);
	    return -1, "createunit ERROR : $pvcreate $params";
	}

	my $vgname;
	my %vg_info = raid_lvm_info_v1::vgdisplay_info();
	my @vgname = sort keys %vg_info;
	my $last_vg = pop @vgname;
	
	if ( defined $last_vg ) {
		if ( $last_vg =~ m/vg(\d+)/ ) {
			$vgname = "vg". ($1 +1 );
		}
	}
	else {
		$vgname = "vg0";
	}

	my $ret2 = qx#$vgcreate $vgname $params 2>&1#;
    
	if ($? ne 0) {
	    return 1, $ret2 if ($?);
	    return -1, "createunit ERROR : $vgcreate $vgname $params";
	}
	$ret .= $ret2;

	my $size = qx#$vgdisplay --colon $vgname | cut -d ":" -f 12#;
	chomp( $size );
	$size .= "k";
	my $nb = scalar @arg;
	my $ret3;

    my @partitions = qx#cat /proc/partitions#;
    my @tab;
    foreach my $disk ( split " ", $params ) {
        $disk =~ s/\/dev\/(.*)/$1/;
        map { if ( m/\s+(\d+)\s+$disk$/ ) { push @tab, $1 } } @partitions;
    }

    my $first;
    my $cmp = 1;
    foreach ( @tab ) {
        if ( ! defined $first ) {
            $first = $_;
            next;
        }
    
        if ( $first != $_ ) {
            $cmp = 0;
            last;
        }
    }

	if ( $cmp ) {
		$ret3 = qx#$lvcreate --size $size --stripes $nb --name lv0 $vgname 2>&1#;
	}
	else {
		$ret3 = qx#$lvcreate --size $size --name lv0 $vgname 2>&1#;
	}
	 
	if ($? ne 0) {
		qx#$vgremove $vgname /dev/null 2>&1#;
	    return 1, $ret3 if ($?);
	    return -1, "createunit ERROR : $vgcreate $vgname $params";
	}
	$ret .= $ret3;


	return 0, $ret;
}

## ###################################
## Fonction deleteunit :
## 
## deleteunit($cid, $uid)

sub deleteunit {
	my $name	= shift;
	my @tab_ctl	= split ("_", $name );
	my $vg		= pop @tab_ctl;
	
	my $lv = shift;
	
    my $ret = qx#$lvremove --force $lv 2>&1#;
    
	if ($? ne 0) {
	    return 1, $ret if ($?);
	    return -1, "deleteunit ERROR : $lvremove $lv";
	}

	my $ret2 = qx#$vgremove $vg 2>&1#;
    
	if ($? ne 0) {
	    return 1, $ret2 if ($?);
	    return -1, "deleteunit ERROR : $vgremove $vg";
	}

	$ret .= $ret2;

   	return 0, $ret if ($? eq 0);
}

##################################
#recuperation des infos smart pour un controleur et un port donne
sub smart {
    my ( $ctl, $unit )	= @_;
    my $ret;

    if ( $unit ) {
        $ret = system( "/sbin/lvs $unit >/dev/null 2>&1" );
    }
    elsif ( $ctl =~ m/^lvm_(.*)$/ ) {
        $ret = system( "/sbin/lvs $1 >/dev/null 2>&1" );
    }
    else {
        return 0, 12;
    }
    
    return 0, 40 if ( $ret );
    return 0, 0;
}

## ###################################
## Fonction state :
sub state {
    return 0, 0;
}

## ###################################
## Fonction capacity :
sub capacity {
    return 0, "raid0";
}

## ###################################
## Fonction rescan :
sub rescan {
	return 0, "Rescan Not Avalaible for lvm";
}

## ###################################
## Fonction verify :
sub verify {
	return 0, "Verify Not Avalaible for lvm";
}

## ###################################
## Fonction alarms :
sub alarms {
	return 0, "";
}

## ###################################
## Fonction rebuild :
sub rebuild {
	return 0, "Rebuild Not Avalaible for lvm";
}

## ###################################
## Fonction deleteport :
sub deleteport {
	return 0, "Deleteport Not Avalaible for lvm";
}


sub _test_lvm_binary {
    my $cmd	= shift;
    my $name	= shift;
    my $func;
    
    chomp($func = `which $cmd`);
    
    if ( ! -e $func){
		$func = $name;
    }

    if ( ! -x $func){
		print "The command $cmd is not executable.\n";
		exit 0;
    }
    return $func;
}

1;
