# --- fichier raid_logiciel_info_v1.pm ---
# Information pour le raid logiciel v1
#############################################################
##
## Made by PUJOS et DAYAN
##
## Started on  Mon July 20 11:14:21 CET 2005 PUJOS et DAYAN
## Last update Tue Oct 17 17:04:32 2006 Pujos Sylvain
##

# nom du package:
package raid_lvm_info_v1;

use strict;
use lib_raid;
use Data::Dumper;

sub pvdisplay_info {
    my $ctl = shift;

    my @file = qx#/sbin/pvdisplay 2> /dev/null#;
    my $hash;
    my %retour;

    return unless ($file[0]);

    foreach (@file) {
		if ( $_ =~ m/---.*Physical volume ---/) {
			if ( $hash->{'PV Name'} and $hash->{'VG Name'} and $hash->{'VG Name'} eq $ctl ) {
				$retour{ $hash->{'PV Name'} } = $hash;
				$hash = undef;
		    }
		} else {
		    $hash->{ $1 } = $2 if ( $_ =~ m/^\s*(\S.*\S)\s\s+(\S.*)$/ );
		}
    }
	
	if ( $hash->{'PV Name'} and $hash->{'VG Name'} and $hash->{'VG Name'} eq $ctl ) {
		$retour{ $hash->{'PV Name'} } = $hash;
		$hash = undef;
    }

    return %retour;
} 

sub vgdisplay_info {
    my $ctl = shift;

    my @file = qx#/sbin/vgdisplay 2> /dev/null#;
    my $hash;
    my %retour;

    return unless ($file[0]);

    foreach (@file) {
		if ( $_ =~ m/--- Volume group ---/) {
			if ( $hash->{'VG Name'} and $hash->{'VG Name'} eq $ctl ) {
				$retour{ $hash->{'VG Name'} } = $hash;
				$hash = undef;
		    }
		} else {
		    $hash->{ $1 } = $2 if ( $_ =~ m/^\s*(\S.*\S)\s\s+(\S.*)$/ );
		}
    }
	
	if ( $hash->{'VG Name'} and $hash->{'VG Name'} eq $ctl ) {
		$retour{ $hash->{'VG Name'} } = $hash;
		$hash = undef;
    }

    return %retour;
}

sub lvdisplay_info {
    my $ctl = shift;

    my @file = qx#/sbin/lvdisplay 2> /dev/null#;
    my $hash;
    my %retour;

    return unless ($file[0]);

    foreach (@file) {
		if ( $_ =~ m/--- Logical volume ---/) {
			if ( $hash->{'LV Name'} and $hash->{'VG Name'} eq $ctl ) {
				$retour{ $hash->{'LV Name'} } = $hash;
				$hash = undef;
		    }
		} else {
		    $hash->{ $1 } = $2 if ( $_ =~ m/^\s*(\S.*\S)\s\s+(\S.*)$/ );
		}
    }
	
	if ( $hash->{'LV Name'} and $hash->{'VG Name'} eq $ctl ) {
		$retour{ $hash->{'LV Name'} } = $hash;
		$hash = undef;
    }

    return %retour;
}

sub info {
    my $ctl = shift;

    my %retour;
    $retour{'Model'} = 'lvm';
    $retour{'Ports'} = '0';
    $retour{'Drives'} = '-';
    $retour{'NotOpt'} = '0';
    $retour{'Units'} = '0';

    my %pv = pvdisplay_info( $ctl );
    return 0, undef unless ( %pv );

    my %lv = lvdisplay_info( $ctl );

    foreach my $unit ( keys %lv ) {
		my %resultat_unit;
		$lv{$unit}{'LV Size'} =~ s/,/\./g ;
	
		$resultat_unit{'Size'} = $1 * 1024	if ( $lv{$unit}{'LV Size'} =~ m/^(.*)\sTB/ );
		$resultat_unit{'Size'} = $1		if ( $lv{$unit}{'LV Size'} =~ m/^(.*)\sGB/ );
	
		$resultat_unit{'Status'}	= 'OK';
		$resultat_unit{'UnitType'}	= 'RAID-0';
		$resultat_unit{'Blocks'}	= '-';
		$resultat_unit{'Cmpl'}		= '-';
        
        $retour{'Units'} = $retour{'Units'} + 1;
	
		$retour{'Unit'}{$unit} = \%resultat_unit;
    }

    foreach my $port ( keys %pv ) {
		my %resultat_port;
	
		if ( $pv{$port}{'VG Name'} ) {
			my @ref = grep { $lv{$_}{'VG Name'} eq $pv{$port}{'VG Name'} } keys %lv;
		
		    $resultat_port{'Unit'} = $ref[0];
		    push @{ $retour{'Unit'}{ $ref[0] }{'Port'} }, $port;
		} else {
		    $resultat_port{'Unit'} = '-';
		    push @{ $retour{'Unit'}{'-'}{'Port'} }, $port;
		}
	
		$pv{$port}{'PV Size'} =~ s/,/\./g ;
		$resultat_port{'Size'} = $1 * 1024	if ( $pv{$port}{'PV Size'} =~ m/^(.*)\sTB/ );
		$resultat_port{'Size'} = $1			if ( $pv{$port}{'PV Size'} =~ m/^(.*)\sGB/ );
		$resultat_port{'Size'} = $1 / 1024	if ( $pv{$port}{'PV Size'} =~ m/^(.*)\sMB/ );
	
		$resultat_port{'Status'} = 'OK';
		$resultat_port{'Blocks'} = '-';
		$resultat_port{'Model'}  = '-';
		$resultat_port{'Serial'} = '-';
        
        $retour{'Ports'} = $retour{'Ports'} + 1;
	
		$retour{'Drive'}{$port} = \%resultat_port;
    }

    my %temp = ( $ctl => \%retour);
    return 0, \%temp;
}


##################################
#recuperation de la liste des ports pour un controleur donne
sub list_port {
	my $name	= shift;
	my @tab_ctl	= split ("_", $name );
	my $ctl		= pop @tab_ctl; 

	my @retour;
    my @file = qx#/sbin/pvdisplay -c 2> /dev/null#;
	return error( undef ) unless ($file[0]);

	foreach (@file) {
		push @retour, $1 if ( m/^\s*(\S+):$ctl:/ );
	}

	return ok( \@retour );
}

##################################
#recuperation de la liste des unites pour un controleur donne
sub list_unit {
	my $name	= shift;
	my @tab_ctl	= split ("_", $name );
	my $ctl		= pop @tab_ctl; 

	my @retour;
    my @file = qx#/sbin/lvdisplay -c 2> /dev/null#;
	return error( undef ) unless ($file[0]);

	foreach (@file) {
		push @retour, $1 if ( m/^\s*(\S+):$ctl:/ );
	}

	return ok( \@retour );
}

1;
