# --- fichier Tw_Cli_info_v2.pm ---
# creation d'un systeme raid sur carte 3Ware
#############################################################
##

# nom du package:
package tw_cli_info_x86_64_v2;

use strict;
use Data::Dumper;
use raid_smart;

use tw_cli_fct_x86_64_v1;

my $tw_cli_name    = "/usr/sbin/tw_cli64";

##################################
#recuperation des infos sur la ou les carte(s) presente(s)
#le hash %tw_cli est retourne
sub tw_cli_info_ctl {
    my $ctl = shift;

    my %tw_cli;
    my %resultat_ctl;
    my $line_ctl = qx#$tw_cli_name info | grep $ctl#;

    return 0, undef unless ( $line_ctl );

# Recuperation des infos comme-ci dessous
# Ctl   Model        Ports   Drives   Units   NotOpt   RRate   VRate

    ($ctl,
     $resultat_ctl{'Model'},
     $resultat_ctl{'Ports'},
     $resultat_ctl{'Drives'},
     $resultat_ctl{'Units'},
     $resultat_ctl{'NotOpt'}) = ($line_ctl =~ m/(\S*)\s*(\S*)\s*(\S*)\s*(\S*)\s*(\S*)\s*(\S*)\s*/);

    $tw_cli{$ctl} = \%resultat_ctl;

# Recuperation des infos sur chaque controleur 
    my @retour_info_ctl = qx#$tw_cli_name info $ctl#;
    @retour_info_ctl = grep {($_ !~ m/.*NOT-PRESENT.*/)} @retour_info_ctl;

    my @liste_unit = grep {($_ =~ m/^u.*/)} @retour_info_ctl;
    my @liste_port = grep {($_ =~ m/^p.*/)} @retour_info_ctl;
    my @list_bbu   = grep {($_ =~ m/^bbu.*/)} @retour_info_ctl;

# Recuperation des infos sur chaque unite comme-ci dessous
# version 2.0.4.15 et plus
# Unit  UnitType  Status         Cmpl  Stripe  Size  Cache  AVerify  OvrECC
# Unit  UnitType  Status         %RCmpl  %V/I/M  Stripe  Size(GB)  Cache  AVrfy

    foreach my $line_unit (@liste_unit) {
        my %resultat_unit;
        my $unit;
        ($unit,
         $resultat_unit{'UnitType'},
         $resultat_unit{'Status'},
         $resultat_unit{'Cmpl'},
         $resultat_unit{'VCmpl'},
         $resultat_unit{'Size'}) = ($line_unit =~ m/(\S*)\s*(\S*)\s*(\S*)\s*(\S*)\s*(\S*)\s*\S*\s*(\S*)\s*/);

        my @retour_info_sub_unit = qx#$tw_cli_name info $ctl $unit#;

        chomp @retour_info_sub_unit;
        map { (m/^u\d+\s.*\s+(\S+)\s*/) ? ($resultat_unit{'Blocks'} = $1) : () } @retour_info_sub_unit;

        $tw_cli{$ctl}{'Unit'}{$unit} = \%resultat_unit;
    }


# Recuperation des infos sur chaque port comme-ci dessous
# Port   Status           Unit   Size        Blocks        Serial
    foreach my $line_port (@liste_port) {
        my %resultat_port;
        my $port;
        my $size_unit;
        my $unit;
        ($port,
         $resultat_port{'Status'},
         $resultat_port{'Unit'},
         $resultat_port{'Size'},
	     $size_unit,
         $resultat_port{'Blocks'},
         $resultat_port{'Serial'}) = ($line_port =~ m/(\S*)\s*(\S*)\s*(\S*)\s*(\S*)\s*(\S*)\s*\S*\s*(\S*)\s*(\S*)/);

	    $resultat_port{'Size'} = $resultat_port{'Size'} * 1024 if ( $size_unit =~ /TB/ );

        $tw_cli{$ctl}{'Drive'}{$port} = \%resultat_port;
        $tw_cli{$ctl}{'Drive'}{$port}{'Model'} = $1 if (qx#$tw_cli_name info $ctl $port model# =~ m/^.*\=\s(.*)/);
        push @{$tw_cli{$ctl}{'Unit'}{ $resultat_port{'Unit'} }{'Port'}}, $port;
    }

    # Recuperation des infos sur chaque bbu comme-ci dessous
    # Name  OnlineState  BBUReady  Status    Volt     Temp     Hours  LastCapTest
    chomp @list_bbu;
    if ( scalar @list_bbu == 1 ) {
        my $bbu;
        my %resultat_bbu;
        ($bbu,
         $resultat_bbu{'OnlineState'},
         $resultat_bbu{'BBUReady'},
    	 $resultat_bbu{'Status'},
	     $resultat_bbu{'Temp'},
    	 $resultat_bbu{'Hours'},
	     $resultat_bbu{'LastCapTest'}) = ($list_bbu[0] =~ m/(\S*)\s*(\S*)\s*(\S*)\s*(\S*)\s*\S*\s*(\S*)\s*(\S*)\s*(\S*)/);

        $tw_cli{$ctl}{'BBU'} = \%resultat_bbu;
    }

    return 0, \%tw_cli;
}

##################################
#recuperation de la liste des controleurs presentes
sub tw_cli_list_ctl {
    my @retour_info = qx#$tw_cli_name info#;
    my @list_ctl;

    # Recuperation des infos comme-ci dessous
    # Ctl   Model        Ports   Drives   Units   NotOpt   RRate   VRate
    foreach my $line_ctl (@retour_info) {
        next unless ( $line_ctl =~ m/^c/);
        next unless ( $line_ctl );

        push @list_ctl, $2 . "_" . $1 if ( $line_ctl =~ m/(\S*)\s*(\S*)\s*/ );
    }

    return 0, \@list_ctl;
}

##################################
#recuperation de la liste des unites pour un controleur donne
sub tw_cli_list_unit {
    my $ctl = shift;

    my @retour_info_ctl = qx#$tw_cli_name info $ctl#;
    @retour_info_ctl = grep {($_ !~ m/.*NOT-PRESENT.*/)} @retour_info_ctl;

    my @liste_unit;
    foreach ( @retour_info_ctl ) {
        push @liste_unit, $1 if ( $_ =~ m/^(u\d*)\s*/ );
    }

    return 0, \@liste_unit;
}

##################################
#recuperation de la liste des ports pour un controleur donne
sub tw_cli_list_port {
    my $ctl = shift;

    my @retour_info_ctl = qx#$tw_cli_name info $ctl#;
    @retour_info_ctl = grep {($_ !~ m/.*NOT-PRESENT.*/)} @retour_info_ctl;

    my @liste_port;
    foreach ( @retour_info_ctl ) {
        push @liste_port, $1 if ( $_ =~ m/^(p\d*)\s*/ );
    }

    return 0, \@liste_port;
}

##################################
#recuperation des infos smart pour un controleur et un port donne
sub tw_cli_smart {
    my ( $ctl, $port ) = @_;

    if ( ! $port ) {
        my ($tmp, $ret) = split ( " = ", 
                qx#$tw_cli_name info $ctl allunitstatus 2> /dev/null | grep "Not Optimal"#
            );

        return 0, 11 unless ($ret);
        $ret =~ s/^\s*(\w*)[\s\n]*$/$1/;

        return 0, 0 if ( $ret eq "0") ;
        return 0, 13;
    }

    if ($port =~ /^u\d+$/) {
        my ($tmp, $ret) = split ( " = ", 
                qx#$tw_cli_name info $ctl $port status 2> /dev/null#
            );

        return 0, 11 unless ($ret);
        $ret =~ s/^\s*(\S*)[\s\n]*$/$1/;
        return 0, raid_smart::translate_text_to_code($ret);
    }

    if ($port =~ /^p(\d+)$/) {
        my ($tmp, $ret) = split ( " = ", 
                qx#$tw_cli_name info $ctl $port status 2> /dev/null#
            );

        return 0, 11 unless ($ret);
        $ret =~ s/^\s*(\S*)[\s\n]*$/$1/;
        return 0, raid_smart::translate_text_to_code($ret);
    }

    if ($port =~ /^bbu$/) {
# status cap lasttest ready
        my (undef, $status) = split ( " = ", qx#$tw_cli_name /$ctl/bbu show status 2> /dev/null# );
        return 0, 11 unless ($status);

        # Si la BBU est Ok, j'active le cache pour chaque unite
        if ( $status =~ m/^OK(\s*\n*)*$/ ) {
            my (undef, $list_unit) = tw_cli_list_unit( $ctl );

            foreach my $unit ( @{$list_unit} ) {
                my (undef, $cache) = tw_cli_fct_x86_64_v1::cache( $ctl, $unit);
		    	if ( $cache and $cache !~ m/=\s*on(\s*\n*)*$/ ) {
                    tw_cli_fct_x86_64_v1::cache( $ctl, $unit, "on" );
	    		}
            }

            my (undef, $lasttest) = split ( " = ", qx#$tw_cli_name /$ctl/bbu show lasttest 2> /dev/null# );
            # Si lasttest est xx-xxx-xxxx, je lance le test
            if ( $lasttest =~ m/xx-xxx-xxxx/ ) {
                tw_cli_fct_x86_64_v1::bbu( $ctl,"test" );
            }

            return 0, 0;
        }
        # Si la BBU est Testing
        if ( $status =~ m/^Testing(\s*\n*)*$/ ) {
            return 0, raid_smart::translate_text_to_code( "Testing" );
        }
        # Si la BBU est Charging
        if ( $status =~ m/^Charging(\s*\n*)*$/ ) {
            my (undef, $lasttest) = split ( " = ", qx#$tw_cli_name /$ctl/bbu show lasttest 2> /dev/null# );
            # Si lasttest est xx-xxx-xxxx, je lance le test
            if ( $lasttest =~ m/xx-xxx-xxxx/ ) {
                tw_cli_fct_x86_64_v1::bbu( $ctl,"test" );
            }

            return 0, raid_smart::translate_text_to_code( "Charging" );
        }

    }

    return 0, "Bad command";
}

## ###################################
## Fonction state :
## 
## state()

sub state {
    my $ctl = shift;
    my $line_ctl = qx#$tw_cli_name /$ctl show allunitstatus | grep Not#;
    return 0, undef unless ( $line_ctl );

    my (undef, $resultat) = split ( "=", $line_ctl );
    return 0, $resultat;
}

1;
