#!/usr/bin/perl
## ######### PROJECT NAME : ##########
## 
## tw_cli_v1.pm
##
## ######### PROJECT DESCRIPTION : ###
## 
## This file contains functions to handle
## 3ware cards.
## 
## ###################################
## 
## Made by husianycia cedric
## Login   <husiana@intellique.com>
## 
## Started on  Wed Aug 31 17:30:31 2005 husianycia cedric
## Last update Fri Mar 21 17:45:54 2008 Boutonnet Alexandre
## 
## ###################################
##

## ###################################
## Declaration du package

package tw_cli_v1;

## ###################################
## Appel des librairies

use	strict;
use	warnings;
use	Data::Dumper;

## Liste des lib info
use	tw_cli_info_i686_v1;
use	tw_cli_info_x86_64_v1;
use	tw_cli_info_i686_v2;
use	tw_cli_info_x86_64_v2;
use	tw_cli_fct_i686_v1;
use	tw_cli_fct_x86_64_v1;

## ###################################
## Declaration de variables


my %up_stage_i686_v1	= (
    'info_ctl'	    => \&tw_cli_info_i686_v1::tw_cli_info_ctl,
    'list_ctl'	    => \&tw_cli_info_i686_v1::tw_cli_list_ctl,
    'list_unit'	    => \&tw_cli_info_i686_v1::tw_cli_list_unit,
    'list_port'	    => \&tw_cli_info_i686_v1::tw_cli_list_port,
    'smart'         => \&tw_cli_info_i686_v1::tw_cli_smart,
    'createunit'    => \&tw_cli_fct_i686_v1::createunit,
    'deleteunit'	=> \&tw_cli_fct_i686_v1::deleteunit,
    'rescan'	    => \&tw_cli_fct_i686_v1::rescan,
    'verify'	    => \&tw_cli_fct_i686_v1::verify,
    'alarms'	    => \&tw_cli_fct_i686_v1::alarms,
    'info'	        => \&tw_cli_fct_i686_v1::info,
    'rebuild'	    => \&tw_cli_fct_i686_v1::rebuild,
    'deleteport'	=> \&tw_cli_fct_i686_v1::deleteport,
    'state'	        => \&tw_cli_info_i686_v1::state,
);

my %up_stage_x86_64_v1	= (
    'info_ctl'	    => \&tw_cli_info_x86_64_v1::tw_cli_info_ctl,
    'list_ctl'	    => \&tw_cli_info_x86_64_v1::tw_cli_list_ctl,
    'list_unit'	    => \&tw_cli_info_x86_64_v1::tw_cli_list_unit,
    'list_port'	    => \&tw_cli_info_x86_64_v1::tw_cli_list_port,
    'smart'	        => \&tw_cli_info_x86_64_v1::tw_cli_smart,
    'createunit'	=> \&tw_cli_fct_x86_64_v1::createunit,
    'deleteunit'	=> \&tw_cli_fct_x86_64_v1::deleteunit,
    'rescan'	    => \&tw_cli_fct_x86_64_v1::rescan,
    'verify'    	=> \&tw_cli_fct_x86_64_v1::verify,
    'alarms'	    => \&tw_cli_fct_x86_64_v1::alarms,
    'info'      	=> \&tw_cli_fct_x86_64_v1::info,
    'rebuild'	    => \&tw_cli_fct_x86_64_v1::rebuild,
    'deleteport'	=> \&tw_cli_fct_x86_64_v1::deleteport,
    'state'	        => \&tw_cli_info_x86_64_v1::state,
);

my %up_stage_i686_v2	= (
    'info_ctl'  	=> \&tw_cli_info_i686_v2::tw_cli_info_ctl,
    'list_ctl'	    => \&tw_cli_info_i686_v2::tw_cli_list_ctl,
    'list_unit' 	=> \&tw_cli_info_i686_v2::tw_cli_list_unit,
    'list_port'	    => \&tw_cli_info_i686_v2::tw_cli_list_port,
    'smart'     	=> \&tw_cli_info_i686_v2::tw_cli_smart,
    'createunit'	=> \&tw_cli_fct_i686_v1::createunit,
    'deleteunit'	=> \&tw_cli_fct_i686_v1::deleteunit,
    'rescan'        => \&tw_cli_fct_i686_v1::rescan,
    'verify'        => \&tw_cli_fct_i686_v1::verify,
    'alarms'        => \&tw_cli_fct_i686_v1::alarms,
    'info'	        => \&tw_cli_fct_i686_v1::info,
    'rebuild'	    => \&tw_cli_fct_i686_v1::rebuild,
    'deleteport'	=> \&tw_cli_fct_i686_v1::deleteport,
    'state'	        => \&tw_cli_info_i686_v2::state,
    'bbu'	        => \&tw_cli_fct_i686_v1::bbu,
    'cache'	        => \&tw_cli_fct_i686_v1::cache,
    'capacity'      => \&tw_cli_info_i686_v2::capacity,
);

my %up_stage_x86_64_v2	= (
    'info_ctl'	    => \&tw_cli_info_x86_64_v2::tw_cli_info_ctl,
    'list_ctl'	    => \&tw_cli_info_x86_64_v2::tw_cli_list_ctl,
    'list_unit' 	=> \&tw_cli_info_x86_64_v2::tw_cli_list_unit,
    'list_port' 	=> \&tw_cli_info_x86_64_v2::tw_cli_list_port,
    'smart'	        => \&tw_cli_info_x86_64_v2::tw_cli_smart,
    'createunit'    => \&tw_cli_fct_x86_64_v1::createunit,
    'deleteunit'	=> \&tw_cli_fct_x86_64_v1::deleteunit,
    'rescan'	    => \&tw_cli_fct_x86_64_v1::rescan,
    'verify'	    => \&tw_cli_fct_x86_64_v1::verify,
    'alarms'	    => \&tw_cli_fct_x86_64_v1::alarms,
    'info'	        => \&tw_cli_fct_x86_64_v1::info,
    'rebuild'	    => \&tw_cli_fct_x86_64_v1::rebuild,
    'deleteport'	=> \&tw_cli_fct_x86_64_v1::deleteport,
    'state'	        => \&tw_cli_info_x86_64_v2::state,
    'bbu'	        => \&tw_cli_fct_x86_64_v1::bbu,
    'cache'	        => \&tw_cli_fct_x86_64_v1::cache,
    'capacity'      => \&tw_cli_info_x86_64_v2::capacity,
);

my %i686    = (
    '2.00.00.015'   => \%up_stage_i686_v1,
    '2.00.00.063'   => \%up_stage_i686_v1,
    '2.00.02.008'   => \%up_stage_i686_v1,
    '2.00.02.009'   => \%up_stage_i686_v1,
    '2.00.03.009'   => \%up_stage_i686_v1,
    '2.00.03.017'   => \%up_stage_i686_v1,
    '2.00.04.015'   => \%up_stage_i686_v2,
    '2.00.06.007'   => \%up_stage_i686_v2,
    '2.00.07.009'   => \%up_stage_i686_v2,
);

my %x86_64  = (
    '2.00.03.017'   => \%up_stage_x86_64_v1,
    '2.00.04.015'   => \%up_stage_x86_64_v2,
    '2.00.06.007'   => \%up_stage_x86_64_v2,
    '2.00.07.009'   => \%up_stage_x86_64_v2,
);

my %architecture	= (
    'i686'      => \%i686,
    'x86_64'    => \%x86_64,
);

my $tw_cli_exe_name	= "/usr/sbin/tw_cli";
my $tw_cli_exe64_name	= "/usr/sbin/tw_cli64";

## ###################################
## ########### FONCTIONS #############
## ###################################

######################################
# appel des fonction ds les librairies
# suivant la version de tw_cli
# prends le nom de la fct et les params
# en parametre

sub call {
    my $name	= shift;
    my @param	= @_;

    my ($version, $arch) = test_version_tw_cli();

    return 1, "tw_cli NOT PRESENT" if ($version eq "1");

    if ( exists $architecture{ $arch } ) {
	if ( exists $architecture{ $arch }-> { $version } ) {
	    if (my @ret = $architecture{$arch}->{$version}->{$name}(@param))
	    {
		return @ret;
	    }
	    else
	    {
		return -1, "tw_cli_info_ctl return null expression";
	    }
	}
	else
	{
	    return -1, "UNKNOWN tw_cli VERSION";
	}
    }
    else
    {
	return -1, "UNKNOWN tw_cli ARCH";
    }

}

######################################
# test la version de tw_cli

sub test_version_tw_cli {
    my $arch	= test_arch();
    my $name	= $tw_cli_exe_name;
    $name = $tw_cli_exe64_name if ($arch eq "x86_64");

    # return 1 si tw_cli n'est pas present
    return 1 unless ( -e $name );
    return 1 unless ( -x $name );

    # On recupere la version de tw_cli et on la retourne
    my $version = qx#$name help | grep version#;
    $version =~ s/^.*version\s(.*)\)\n$/$1/;

    return ($version, $arch);
}

######################################
# test la version de tw_cli

sub test_arch {

    # On recupere la version de tw_cli et on la retourne
    my $arch = qx#uname -m#;
    chomp($arch);

    return $arch;
}

## ###################################
## Fonction info_all :
## appel tw_cli_info et retourne les infos
## si la version est connue

sub info_all {
    return call("info_ctl", @_);
}

## ###################################
## Fonction list_ctl :

sub list_ctl {

    return call("list_ctl", @_);
}

## ###################################
## Fonction list_unit :
## prend un controleur en param

sub list_unit {
    return call("list_unit", @_);
}

## ###################################
## Fonction list_port :
## prend un controleur en param

sub list_port {
    return call("list_port", @_);
}

## ###################################
## Fonction smart :
## prend un controleur et un disk en param

sub smart {
    return call("smart", @_);
}

## ###################################
## Fonction createunit :
##
## createunit($cid, $RaidType, $PidList)

sub createunit {
    return call("createunit", @_);
}

## ###################################
## Fonction deleteunit :
## 
## deleteunit($cid, $uid)

sub deleteunit {
    return call("deleteunit", @_);
}

## ###################################
## Fonction rescan :
## 
## rescan($cid)

sub rescan {
    return call("rescan", @_);
}

## ###################################
## Fonction verify :
## 
## verify($cid, $uid)

sub verify {
   return call("verify", @_);
}

## ###################################
## Fonction alarms :
## 
## alarms([$cid]) 

sub alarms {
    return call("alarms", @_);
}

## ###################################
## Fonction info :
## 
## info()

sub info {
    return call("info", @_);
}

## ###################################
## Fonction rebuilt :
## 
## rebuild($cid, $uid, $pid)

sub rebuild {
    return call("rebuild", @_);
}

## ###################################
## Fonction deleteport :
## 
## deleteport($cid, $pid)

sub deleteport {
    return call("deleteport", @_);
}


## ###################################
## Fonction state :
## 
## state()

sub state {
    return call("state", @_);
}


## ###################################
## Fonction bbu :
sub bbu {
    return call("bbu", @_);
}

## ###################################
## Fonction cache :
sub cache {
    return call("cache", @_);
}

## ###################################
## Fonction capacity :
sub capacity {
    return call("capacity", @_);
}

1;
